# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'analyse_dialog_ui.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_AnalyseDialog(object):
    def setupUi(self, AnalyseDialog):
        AnalyseDialog.setObjectName("AnalyseDialog")
        AnalyseDialog.resize(791, 638)
        self.plot = QtWidgets.QWidget(AnalyseDialog)
        self.plot.setGeometry(QtCore.QRect(0, 130, 791, 501))
        font = QtGui.QFont()
        font.setFamily("Arial")
        self.plot.setFont(font)
        self.plot.setObjectName("plot")
        self.btn_selectDecError = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectDecError.setGeometry(QtCore.QRect(10, 10, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectDecError.setFont(font)
        self.btn_selectDecError.setObjectName("btn_selectDecError")
        self.btn_selectRaError = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectRaError.setGeometry(QtCore.QRect(140, 10, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectRaError.setFont(font)
        self.btn_selectRaError.setObjectName("btn_selectRaError")
        self.btn_selectDecErrorAltitude = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectDecErrorAltitude.setGeometry(QtCore.QRect(270, 10, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectDecErrorAltitude.setFont(font)
        self.btn_selectDecErrorAltitude.setObjectName("btn_selectDecErrorAltitude")
        self.btn_selectRaErrorAltitude = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectRaErrorAltitude.setGeometry(QtCore.QRect(400, 10, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectRaErrorAltitude.setFont(font)
        self.btn_selectRaErrorAltitude.setObjectName("btn_selectRaErrorAltitude")
        self.btn_selectModelPointPolar = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectModelPointPolar.setGeometry(QtCore.QRect(530, 10, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectModelPointPolar.setFont(font)
        self.btn_selectModelPointPolar.setObjectName("btn_selectModelPointPolar")
        self.btn_selectDecErrorAzimuth = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectDecErrorAzimuth.setGeometry(QtCore.QRect(270, 50, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectDecErrorAzimuth.setFont(font)
        self.btn_selectDecErrorAzimuth.setObjectName("btn_selectDecErrorAzimuth")
        self.btn_selectModelPointErrorPolar = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectModelPointErrorPolar.setGeometry(QtCore.QRect(530, 50, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectModelPointErrorPolar.setFont(font)
        self.btn_selectModelPointErrorPolar.setObjectName("btn_selectModelPointErrorPolar")
        self.btn_selectRaErrorDeviation = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectRaErrorDeviation.setGeometry(QtCore.QRect(140, 50, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectRaErrorDeviation.setFont(font)
        self.btn_selectRaErrorDeviation.setObjectName("btn_selectRaErrorDeviation")
        self.btn_selectRaErrorAzimuth = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectRaErrorAzimuth.setGeometry(QtCore.QRect(400, 50, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectRaErrorAzimuth.setFont(font)
        self.btn_selectRaErrorAzimuth.setObjectName("btn_selectRaErrorAzimuth")
        self.label_93 = QtWidgets.QLabel(AnalyseDialog)
        self.label_93.setGeometry(QtCore.QRect(140, 100, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_93.setFont(font)
        self.label_93.setObjectName("label_93")
        self.label_113 = QtWidgets.QLabel(AnalyseDialog)
        self.label_113.setGeometry(QtCore.QRect(240, 100, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_113.setFont(font)
        self.label_113.setAlignment(QtCore.Qt.AlignCenter)
        self.label_113.setWordWrap(False)
        self.label_113.setObjectName("label_113")
        self.scalePlotRA = QtWidgets.QDoubleSpinBox(AnalyseDialog)
        self.scalePlotRA.setGeometry(QtCore.QRect(200, 100, 41, 22))
        self.scalePlotRA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.scalePlotRA.setDecimals(0)
        self.scalePlotRA.setMinimum(5.0)
        self.scalePlotRA.setMaximum(100.0)
        self.scalePlotRA.setSingleStep(5.0)
        self.scalePlotRA.setProperty("value", 20.0)
        self.scalePlotRA.setObjectName("scalePlotRA")
        self.label_94 = QtWidgets.QLabel(AnalyseDialog)
        self.label_94.setGeometry(QtCore.QRect(10, 100, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_94.setFont(font)
        self.label_94.setObjectName("label_94")
        self.scalePlotDEC = QtWidgets.QDoubleSpinBox(AnalyseDialog)
        self.scalePlotDEC.setGeometry(QtCore.QRect(80, 100, 41, 22))
        self.scalePlotDEC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.scalePlotDEC.setDecimals(0)
        self.scalePlotDEC.setMinimum(5.0)
        self.scalePlotDEC.setMaximum(100.0)
        self.scalePlotDEC.setSingleStep(5.0)
        self.scalePlotDEC.setProperty("value", 20.0)
        self.scalePlotDEC.setObjectName("scalePlotDEC")
        self.label_114 = QtWidgets.QLabel(AnalyseDialog)
        self.label_114.setGeometry(QtCore.QRect(120, 100, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_114.setFont(font)
        self.label_114.setAlignment(QtCore.Qt.AlignCenter)
        self.label_114.setWordWrap(False)
        self.label_114.setObjectName("label_114")
        self.label_95 = QtWidgets.QLabel(AnalyseDialog)
        self.label_95.setGeometry(QtCore.QRect(270, 100, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_95.setFont(font)
        self.label_95.setObjectName("label_95")
        self.label_115 = QtWidgets.QLabel(AnalyseDialog)
        self.label_115.setGeometry(QtCore.QRect(380, 100, 51, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_115.setFont(font)
        self.label_115.setAlignment(QtCore.Qt.AlignCenter)
        self.label_115.setWordWrap(False)
        self.label_115.setObjectName("label_115")
        self.scalePlotError = QtWidgets.QDoubleSpinBox(AnalyseDialog)
        self.scalePlotError.setGeometry(QtCore.QRect(340, 100, 41, 22))
        self.scalePlotError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.scalePlotError.setDecimals(0)
        self.scalePlotError.setMinimum(5.0)
        self.scalePlotError.setMaximum(50.0)
        self.scalePlotError.setSingleStep(5.0)
        self.scalePlotError.setProperty("value", 10.0)
        self.scalePlotError.setObjectName("scalePlotError")
        self.btn_selectDecErrorDeviation = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectDecErrorDeviation.setGeometry(QtCore.QRect(10, 50, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectDecErrorDeviation.setFont(font)
        self.btn_selectDecErrorDeviation.setObjectName("btn_selectDecErrorDeviation")
        self.btn_selectAnalyseFileName_11 = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectAnalyseFileName_11.setGeometry(QtCore.QRect(660, 10, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectAnalyseFileName_11.setFont(font)
        self.btn_selectAnalyseFileName_11.setObjectName("btn_selectAnalyseFileName_11")
        self.btn_selectAnalyseFileName_12 = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_selectAnalyseFileName_12.setGeometry(QtCore.QRect(660, 50, 121, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectAnalyseFileName_12.setFont(font)
        self.btn_selectAnalyseFileName_12.setObjectName("btn_selectAnalyseFileName_12")

        self.retranslateUi(AnalyseDialog)
        QtCore.QMetaObject.connectSlotsByName(AnalyseDialog)

    def retranslateUi(self, AnalyseDialog):
        _translate = QtCore.QCoreApplication.translate
        AnalyseDialog.setWindowTitle(_translate("AnalyseDialog", "Analyse"))
        self.btn_selectDecError.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show Dec Error in arc sec from modeling file</p></body></html>"))
        self.btn_selectDecError.setText(_translate("AnalyseDialog", "Dec Error"))
        self.btn_selectRaError.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show Ra Error in arc sec from modeling file.</p></body></html>"))
        self.btn_selectRaError.setText(_translate("AnalyseDialog", "Ra Error"))
        self.btn_selectDecErrorAltitude.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show Dec Error in arc sec from modeling file over Altitude.</p></body></html>"))
        self.btn_selectDecErrorAltitude.setText(_translate("AnalyseDialog", "Dec Error over Alt"))
        self.btn_selectRaErrorAltitude.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show Ra Error in arc sec from modeling file over Altitude.</p></body></html>"))
        self.btn_selectRaErrorAltitude.setText(_translate("AnalyseDialog", "Ra Error over Alt"))
        self.btn_selectModelPointPolar.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show model points in colors over altitude and azimuth in polar diagram. blue: points west, green: points east.</p></body></html>"))
        self.btn_selectModelPointPolar.setText(_translate("AnalyseDialog", "Model Points Polar"))
        self.btn_selectDecErrorAzimuth.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show Dec Error in arc sec from modeling file over Azimuth.</p></body></html>"))
        self.btn_selectDecErrorAzimuth.setText(_translate("AnalyseDialog", "Dec Error over Az"))
        self.btn_selectModelPointErrorPolar.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show total error in colors over altitude and azimuth in polar diagram. blue: points west, green: points east.</p></body></html>"))
        self.btn_selectModelPointErrorPolar.setText(_translate("AnalyseDialog", "Model Error Polar"))
        self.btn_selectRaErrorDeviation.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show Ra Error in arc sec from modeling file with respect to the first model point.</p></body></html>"))
        self.btn_selectRaErrorDeviation.setText(_translate("AnalyseDialog", "RA Error Flexure"))
        self.btn_selectRaErrorAzimuth.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show Ra Error in arc sec from modeling file over Azimuth.</p></body></html>"))
        self.btn_selectRaErrorAzimuth.setText(_translate("AnalyseDialog", "Ra Error over Az"))
        self.label_93.setText(_translate("AnalyseDialog", "Scale RA"))
        self.label_113.setText(_translate("AnalyseDialog", "°"))
        self.scalePlotRA.setToolTip(_translate("AnalyseDialog", "resulting focal length - please think of reducers etc."))
        self.label_94.setText(_translate("AnalyseDialog", "Scale DEC"))
        self.scalePlotDEC.setToolTip(_translate("AnalyseDialog", "resulting focal length - please think of reducers etc."))
        self.label_114.setText(_translate("AnalyseDialog", "°"))
        self.label_95.setText(_translate("AnalyseDialog", "Scale Error"))
        self.label_115.setText(_translate("AnalyseDialog", "arcsec"))
        self.scalePlotError.setToolTip(_translate("AnalyseDialog", "resulting focal length - please think of reducers etc."))
        self.btn_selectDecErrorDeviation.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Show Dec Error in arc sec from modeling file with respect to the first model point.</p></body></html>"))
        self.btn_selectDecErrorDeviation.setText(_translate("AnalyseDialog", "DEC Error Flexure"))
        self.btn_selectAnalyseFileName_11.setToolTip(_translate("AnalyseDialog", "Sets dual tracking on / off"))
        self.btn_selectAnalyseFileName_11.setText(_translate("AnalyseDialog", "..."))
        self.btn_selectAnalyseFileName_12.setToolTip(_translate("AnalyseDialog", "Sets dual tracking on / off"))
        self.btn_selectAnalyseFileName_12.setText(_translate("AnalyseDialog", "..."))




############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
import platform
import json
import logging
import time
# packages for handling web interface to SGPro
from urllib import request
from baseclasses.camera import MWCamera


class SGPro(MWCamera):
    logger = logging.getLogger(__name__)                                                                                    # logging enabling

    def __init__(self, app):
        super(SGPro, self).__init__(app)
        self.host = '127.0.0.1'
        self.port = 59590
        self.ipSGProBase = 'http://' + self.host + ':' + str(self.port)
        self.ipSGPro = 'http://' + self.host + ':' + str(self.port) + '/json/reply/'
        self.captureImagePath = 'SgCaptureImage'
        self.connectDevicePath = 'SgConnectDevicePath'
        self.disconnectDevicePath = 'SgDisconnectDevicePath'
        self.getCameraPropsPath = 'SgGetCameraProps'
        self.getDeviceStatusPath = 'SgGetDeviceStatus'
        self.enumerateDevicePath = 'SgEnumerateDevices'
        self.getImagePath = 'SgGetImagePath'
        self.getSolvedImageDataPath = 'SgGetSolvedImageData'
        self.solveImagePath = 'SgSolveImage'
        self.appExe = 'Sequence Generator.exe'
        self.checkAppInstall()

    def checkAppInstall(self):
        if platform.system() == 'Windows':
            self.appAvailable, self.appName, self.appInstallPath = self.app.checkRegistrationKeys('Sequence Generator')
            if self.appAvailable:
                self.app.messageQueue.put('Found: {0}'.format(self.appName))
                self.logger.info('Name: {0}, Path: {1}'.format(self.appName, self.appInstallPath))
            else:
                self.logger.info('Application SGPro not found on computer')

    def connectCamera(self):
        if self.appRunning:
            pass

    def disconnectCamera(self):
        if self.appRunning:
            self.cameraConnected = False

    def getImage(self, modelData):
        suc, mes, guid = self.SgCaptureImage(binningMode=modelData['Binning'],
                                             exposureLength=modelData['Exposure'],
                                             iso=str(modelData['Iso']),
                                             gain=modelData['GainValue'],
                                             speed=modelData['Speed'],
                                             frameType='Light',
                                             filename=modelData['File'],
                                             path=modelData['BaseDirImages'],
                                             useSubframe=modelData['CanSubframe'],
                                             posX=modelData['OffX'],
                                             posY=modelData['OffY'],
                                             width=modelData['SizeX'],
                                             height=modelData['SizeY'])
        modelData['ImagePath'] = ''
        self.logger.info('message: {0}'.format(mes))
        if suc:                                                                                                             # if we successfully starts imaging, we ca move on
            while True:                                                                                                     # waiting for the image download before proceeding
                suc, modelData['ImagePath'] = self.SgGetImagePath(guid)                                                     # there is the image path, once the image is downloaded
                if suc:                                                                                                     # until then, the link is only the receipt
                    break                                                                                                   # stopping the loop
                else:                                                                                                       # otherwise
                    time.sleep(0.5)                                                                                         # wait for 0.5 seconds
        return suc, mes, modelData

    def solveImage(self, modelData):
        suc, mes, guid = self.SgSolveImage(modelData['ImagePath'],
                                           scaleHint=modelData['ScaleHint'],
                                           blindSolve=modelData['Blind'],
                                           useFitsHeaders=modelData['UseFitsHeaders'])
        if not suc:
            self.logger.warning('no start {0}'.format(mes))
            return False, mes, modelData
        while True:                                                                                                         # retrieving solving data in loop
            suc, mes, ra_sol, dec_sol, scale, angle, timeTS = self.SgGetSolvedImageData(guid)                               # retrieving the data from solver
            mes = mes.strip('\n')                                                                                           # sometimes there are heading \n in message
            if mes[:7] in ['Matched', 'Solve t', 'Valid s', 'succeed']:                                                     # if there is success, we can move on
                self.logger.info('modelData {0}'.format(modelData))
                solved = True
                modelData['RaJ2000Solved'] = float(ra_sol)
                modelData['DecJ2000Solved'] = float(dec_sol)                                                                       # convert values to float, should be stored in float not string
                modelData['Scale'] = float(scale)
                modelData['Angle'] = float(angle)
                modelData['TimeTS'] = float(timeTS)
                break
            elif mes != 'Solving':                                                                                          # general error
                solved = False
                break
            # TODO: clarification should we again introduce model run cancel during plate solving -> very complicated solver should cancel if not possible after some time
            # elif app.model.cancel:
            #    solved = False
            #    break
            else:                                                                                                           # otherwise
                if modelData['Blind']:                                                                                      # when using blind solve, it takes 30-60 s
                    time.sleep(5)                                                                                           # therefore slow cycle
                else:                                                                                                       # local solver takes 1-2 s
                    time.sleep(.25)                                                                                         # therefore quicker cycle
        return solved, mes, modelData

    def getCameraProps(self):
        return self.SgGetCameraProps()

    def getCameraStatus(self):
        if self.appRunning:
            suc, mes = self.SgGetDeviceStatus('Camera')
            if suc:
                self.cameraConnected = True
                if mes == 'IDLE':
                    self.cameraStatus = 'READY - IDLE'
                elif mes == 'CAPTURING':
                    self.cameraStatus = 'INTEGRATING'
                elif mes == 'BUSY':
                    self.cameraStatus = 'DOWNLOADING'
                elif mes == 'DISCONNECTED':
                    self.cameraStatus = 'DISCONNECTED'
                    self.cameraConnected = False
            else:
                self.cameraStatus = 'ERROR'
                self.cameraConnected = False

    def SgCaptureImage(self, binningMode=1, exposureLength=1,
                       gain=None, iso=None, speed=None, frameType=None, filename=None,
                       path=None, useSubframe=False, posX=0, posY=0,
                       width=1, height=1):
        # reference {"BinningMode":0,"ExposureLength":0,"Gain":"String","Speed":"Normal","FrameType":"Light",
        # reference "Path":"String","UseSubframe":false,"X":0,"Y":0,"Width":0,"Height":0}
        data = {"BinningMode": binningMode, "ExposureLength": exposureLength, "UseSubframe": useSubframe, "X": posX, "Y": posY,
                "Width": width, "Height": height}
        if gain:
            data['Gain'] = gain
        if iso:
            data['Iso'] = iso
        if speed:
            data['Speed'] = speed
        if frameType:
            data['FrameType'] = frameType
        if path and filename:
            data['Path'] = path + '/' + filename
        try:
            req = request.Request(self.ipSGPro + self.captureImagePath, data=bytes(json.dumps(data).encode('utf-8')), method='POST')
            req.add_header('Content-Type', 'application/json')
            with request.urlopen(req) as f:
                captureResponse = json.loads(f.read().decode('utf-8'))
            # {"Success":false,"Message":"String","Receipt":"00000000000000000000000000000000"}
            return captureResponse['Success'], captureResponse['Message'], captureResponse['Receipt']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', ''

    def SgGetCameraProps(self):
        # reference {}
        data = {}
        try:
            req = request.Request(self.ipSGPro + self.getCameraPropsPath, data=bytes(json.dumps(data).encode('utf-8')), method='POST')
            req.add_header('Content-Type', 'application/json')
            with request.urlopen(req) as f:
                captureResponse = json.loads(f.read().decode('utf-8'))
            # {"Success":false,"Message":"String","NumPixelsX":0,"NumPixelsY":0,"SupportsSubframe":false}
            if 'GainValues' not in captureResponse:
                captureResponse['GainValues'] = ['High']
            return captureResponse['Success'], captureResponse['Message'], int(captureResponse['NumPixelsX']), int(captureResponse['NumPixelsY']), captureResponse['SupportsSubframe'], captureResponse['GainValues'][0]
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', '', '', ''

    def SgGetDeviceStatus(self, device):
        # reference {"Device": "Camera"}, devices are "Camera", "FilterWheel", "Focuser", "Telescope" and "PlateSolver"}
        data = {'Device': device}
        try:
            req = request.Request(self.ipSGPro + self.getDeviceStatusPath, data=bytes(json.dumps(data).encode('utf-8')), method='POST')
            req.add_header('Content-Type', 'application/json')
            with request.urlopen(req) as f:
                captureResponse = json.loads(f.read().decode('utf-8'))
            # states are  "IDLE", "CAPTURING", "BUSY", "MOVING", "DISCONNECTED", "PARKED"
            # {"State":"IDLE","Success":false,"Message":"String"}
            return captureResponse['Success'], captureResponse['State']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed'

    def SgGetImagePath(self, _guid):
        # reference {"Receipt":"00000000000000000000000000000000"}
        data = {'Receipt': _guid}
        try:
            req = request.Request(self.ipSGPro + self.getImagePath, data=bytes(json.dumps(data).encode('utf-8')), method='POST')
            req.add_header('Content-Type', 'application/json')
            with request.urlopen(req) as f:
                captureResponse = json.loads(f.read().decode('utf-8'))
            # {"Success":false,"Message":"String"}
            return captureResponse['Success'], captureResponse['Message']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed'

    def SgGetSolvedImageData(self, _guid):
        # reference {"Receipt":"00000000000000000000000000000000"}
        data = {'Receipt': _guid}
        try:
            req = request.Request(self.ipSGPro + self.getSolvedImageDataPath, data=bytes(json.dumps(data).encode('utf-8')), method='POST')
            req.add_header('Content-Type', 'application/json')
            with request.urlopen(req) as f:
                captureResponse = json.loads(f.read().decode('utf-8'))
            # {"Success":false,"Message":"String","Ra":0,"Dec":0,"Scale":0,"Angle":0,"TimeToSolve":0}
            return captureResponse['Success'], captureResponse['Message'], captureResponse['Ra'], captureResponse['Dec'], captureResponse['Scale'], captureResponse['Angle'], captureResponse['TimeToSolve']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', '', '', '', '', ''

    def SgSolveImage(self, path, raHint=None, decHint=None, scaleHint=None, blindSolve=False, useFitsHeaders=False):
        # reference {"ImagePath":"String","RaHint":0,"DecHint":0,"ScaleHint":0,"BlindSolve":false,"UseFitsHeadersForHints":false}
        data = {"ImagePath": path, "BlindSolve": blindSolve, "UseFitsHeadersForHints": useFitsHeaders}
        if raHint:
            data['RaHint'] = raHint
        if decHint:
            data['DecHint'] = decHint
        if scaleHint:
            data['ScaleHint'] = scaleHint
        try:
            req = request.Request(self.ipSGPro + self.solveImagePath, data=bytes(json.dumps(data).encode('utf-8')), method='POST')
            req.add_header('Content-Type', 'application/json')
            with request.urlopen(req) as f:
                captureResponse = json.loads(f.read().decode('utf-8'))                                                      # {"Success":false,"Message":"String","Receipt":"00000000000000000000000000000000"}
            return captureResponse['Success'], captureResponse['Message'], captureResponse['Receipt']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', ''


if __name__ == "__main__":
    cam = SGPro(MWCamera)
    print(cam.getCameraProps())
    '''
    value = {'Binning': 1, 'Exposure': 1, 'Iso': 100,
             'GainValue': 'Not Set', 'Speed': 'HiSpeed',
             'File': 'test.fit', 'BaseDirImages': 'c:/temp',
             'CanSubframe': True, 'OffX': 0, 'OffY': 0,
             'SizeX': 3388, 'SizeY': 2712}
    '''
    suci, msgi, guidi = cam.SgCaptureImage(binningMode=1, exposureLength=10, gain='Not Set', iso=100, speed='HighSpeed', frameType='Light', path='c:\\temp\\', filename= 'test.fit', useSubframe=False, posX=0, posY=0, width=1, height=1)
    suc, msg, guid = cam.SgSolveImage(path='C:\\Users\\mw\\Projects\\mountwizzard\\mountwizzard\\model001.fit', scaleHint=1.3, useFitsHeaders=True)
    print(suci, msgi, guidi, suc, msg, guid)
    while True:
        print(cam.SgGetImagePath(guidi))
        print(cam.SgGetSolvedImageData(guid))
        time.sleep(1)



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################

# import basic stuff
import os
import logging
import time
# PyQt5
import PyQt5
# webservices
import urllib.request as urllib2
# windows automation
from pywinauto import Application, timings, findwindows, application
from pywinauto.controls.win32_controls import ButtonWrapper, EditWrapper


class DataUploadToMount(PyQt5.QtCore.QThread):
    logger = logging.getLogger(__name__)                                                                                    # get logger for  problems

    UTC_1 = 'http://maia.usno.navy.mil/ser7/finals.data'
    UTC_2 = 'http://maia.usno.navy.mil/ser7/tai-utc.dat'
    COMETS = 'http://www.minorplanetcenter.net/iau/MPCORB/CometEls.txt'
    COMETS_START = 102
    COMETS_END = 161
    ASTEROIDS_START = 165
    ASTEROIDS_END = 196
    ASTEROIDS_MPC5000 = 'http://www.ap-i.net/pub/skychart/mpc/mpc5000.dat'
    # ASTEROIDS = 'http://www.minorplanetcenter.net/iau/MPCORB/MPCORB.DAT'
    ASTEROIDS_NEA = 'http://www.minorplanetcenter.net/iau/MPCORB/NEA.txt'
    ASTEROIDS_PHA = 'http://www.minorplanetcenter.net/iau/MPCORB/PHA.txt'
    ASTEROIDS_TNO = 'http://www.minorplanetcenter.net/iau/MPCORB/Distant.txt'
    ASTEROIDS_UNUSAL = 'http://www.minorplanetcenter.net/iau/MPCORB/Unusual.txt'
    SPACESTATIONS = 'http://www.celestrak.com/NORAD/elements/stations.txt'
    SATBRIGHTEST = 'http://www.celestrak.com/NORAD/elements/visual.txt'
    TARGET_DIR = os.getcwd() + '\\'
    COMETS_FILE = 'comets.mpc'
    ASTEROIDS_FILE = 'asteroids.mpc'
    SPACESTATIONS_FILE = 'spacestations.tle'
    SATBRIGHTEST_FILE = 'satbrightest.tle'
    UTC_1_FILE = 'finals.data'
    UTC_2_FILE = 'tai-utc.dat'
    BLUE = 'background-color: rgb(42, 130, 218)'
    RED = 'background-color: red;'
    DEFAULT = 'background-color: rgb(32,32,32); color: rgb(192,192,192)'
    OPENDIALOG = 'Dialog'

    def __init__(self, app):
        super().__init__()
        self.app = app
        self.appAvailable = False
        self.appName = ''
        self.appInstallPath = ''
        self.appExe = 'GmQCIv2.exe'
        self.app.ui.btn_downloadEarthrotation.clicked.connect(lambda: self.app.commandDataQueue.put('EARTHROTATION'))
        self.app.ui.btn_downloadSpacestations.clicked.connect(lambda: self.app.commandDataQueue.put('SPACESTATIONS'))
        self.app.ui.btn_downloadSatbrighest.clicked.connect(lambda: self.app.commandDataQueue.put('SATBRIGHTEST'))
        self.app.ui.btn_downloadAsteroidsMPC5000.clicked.connect(lambda: self.app.commandDataQueue.put('ASTEROIDS_MPC5000'))
        self.app.ui.btn_downloadAsteroidsNEA.clicked.connect(lambda: self.app.commandDataQueue.put('ASTEROIDS_NEA'))
        self.app.ui.btn_downloadAsteroidsPHA.clicked.connect(lambda: self.app.commandDataQueue.put('ASTEROIDS_PHA'))
        self.app.ui.btn_downloadAsteroidsTNO.clicked.connect(lambda: self.app.commandDataQueue.put('ASTEROIDS_TNO'))
        self.app.ui.btn_downloadComets.clicked.connect(lambda: self.app.commandDataQueue.put('COMETS'))
        self.app.ui.btn_downloadAll.clicked.connect(lambda: self.app.commandDataQueue.put('ALL'))
        self.app.ui.btn_uploadMount.clicked.connect(lambda: self.app.commandDataQueue.put('UPLOADMOUNT'))
        self.checkApplication()
        self.TARGET_DIR = self.appInstallPath
        self.initConfig()

    def initConfig(self):
        try:
            if 'FilterExpressionMPC' in self.app.config:
                self.app.ui.le_filterExpressionMPC.setText(self.app.config['FilterExpressionMPC'])
            if 'CheckFilterMPC' in self.app.config:
                self.app.ui.checkFilterMPC.setChecked(self.app.config['CheckFilterMPC'])
        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass

    def storeConfig(self):
        self.app.config['CheckFilterMPC'] = self.app.ui.checkFilterMPC.isChecked()
        self.app.config['FilterExpressionMPC'] = self.app.ui.le_filterExpressionMPC.text()

    def checkApplication(self):
        self.appAvailable, self.appName, self.appInstallPath = self.app.checkRegistrationKeys('10micron QCI')
        if self.appAvailable:
            self.app.messageQueue.put('Found: {0}'.format(self.appName))
            self.logger.info('Name: {0}, Path: {1}'.format(self.appName, self.appInstallPath))
        else:
            self.logger.info('Application 10micron Updater  not found on computer')

    def run(self):                                                                                                          # runnable for doing the work
        while True:                                                                                                         # main loop for stick thread
            if not self.app.commandDataQueue.empty():
                command = self.app.commandDataQueue.get()
                if command == 'SPACESTATIONS':
                    self.app.ui.btn_downloadSpacestations.setStyleSheet(self.BLUE)
                    self.downloadFile(self.SPACESTATIONS, self.TARGET_DIR + self.SPACESTATIONS_FILE)
                    self.app.ui.checkSpacestations.setChecked(True)
                    self.app.ui.btn_downloadSpacestations.setStyleSheet(self.DEFAULT)
                elif command == 'SATBRIGHTEST':
                    self.app.ui.btn_downloadSatbrighest.setStyleSheet(self.BLUE)
                    self.downloadFile(self.SATBRIGHTEST, self.TARGET_DIR + self.SATBRIGHTEST_FILE)
                    self.app.ui.checkSatellites.setChecked(True)
                    self.app.ui.btn_downloadSatbrighest.setStyleSheet(self.DEFAULT)
                elif command == 'ASTEROIDS_MPC5000':
                    self.app.ui.btn_downloadAsteroidsMPC5000.setStyleSheet(self.BLUE)
                    self.downloadFile(self.ASTEROIDS_MPC5000, self.TARGET_DIR + self.ASTEROIDS_FILE)
                    self.app.ui.checkAsteroids.setChecked(True)
                    self.app.ui.btn_downloadAsteroidsMPC5000.setStyleSheet(self.DEFAULT)
                elif command == 'ASTEROIDS_NEA':
                    self.app.ui.btn_downloadAsteroidsNEA.setStyleSheet(self.BLUE)
                    self.downloadFile(self.ASTEROIDS_NEA, self.TARGET_DIR + self.ASTEROIDS_FILE)
                    self.app.ui.checkAsteroids.setChecked(True)
                    self.app.ui.btn_downloadAsteroidsNEA.setStyleSheet(self.DEFAULT)
                elif command == 'ASTEROIDS_PHA':
                    self.app.ui.btn_downloadAsteroidsPHA.setStyleSheet(self.BLUE)
                    self.downloadFile(self.ASTEROIDS_PHA, self.TARGET_DIR + self.ASTEROIDS_FILE)
                    self.app.ui.checkAsteroids.setChecked(True)
                    self.app.ui.btn_downloadAsteroidsPHA.setStyleSheet(self.DEFAULT)
                elif command == 'ASTEROIDS_TNO':
                    self.app.ui.btn_downloadAsteroidsTNO.setStyleSheet(self.BLUE)
                    self.downloadFile(self.ASTEROIDS_TNO, self.TARGET_DIR + self.ASTEROIDS_FILE)
                    self.app.ui.checkAsteroids.setChecked(True)
                    self.app.ui.btn_downloadAsteroidsTNO.setStyleSheet(self.DEFAULT)
                elif command == 'ASTEROIDS_UNUSAL':
                    self.app.ui.btn_downloadAsteroidsUNUSAL.setStyleSheet(self.BLUE)
                    self.downloadFile(self.ASTEROIDS_UNUSAL, self.TARGET_DIR + self.ASTEROIDS_FILE)
                    self.app.ui.checkAsteroids.setChecked(True)
                    self.app.ui.btn_downloadAsteroidsUNUSAL.setStyleSheet(self.DEFAULT)
                elif command == 'COMETS':
                    self.app.ui.btn_downloadComets.setStyleSheet(self.BLUE)
                    self.downloadFile(self.COMETS, self.TARGET_DIR + self.COMETS_FILE)
                    self.app.ui.checkComets.setChecked(True)
                    self.app.ui.btn_downloadComets.setStyleSheet(self.DEFAULT)
                elif command == 'EARTHROTATION':
                    self.app.ui.btn_downloadEarthrotation.setStyleSheet(self.BLUE)
                    self.downloadFile(self.UTC_1, self.TARGET_DIR + self.UTC_1_FILE)
                    self.downloadFile(self.UTC_2, self.TARGET_DIR + self.UTC_2_FILE)
                    self.app.ui.checkEarthrotation.setChecked(True)
                    self.app.ui.btn_downloadEarthrotation.setStyleSheet(self.DEFAULT)
                elif command == 'ALL':
                    self.app.ui.btn_downloadAll.setStyleSheet(self.BLUE)
                    self.app.ui.btn_downloadEarthrotation.setStyleSheet(self.BLUE)
                    self.app.ui.btn_downloadSpacestations.setStyleSheet(self.BLUE)
                    self.app.ui.btn_downloadSatbrighest.setStyleSheet(self.BLUE)
                    self.app.ui.btn_downloadAsteroidsMPC5000.setStyleSheet(self.BLUE)
                    self.app.ui.btn_downloadComets.setStyleSheet(self.BLUE)
                    self.downloadFile(self.UTC_1, self.TARGET_DIR + self.UTC_1_FILE)
                    self.downloadFile(self.UTC_2, self.TARGET_DIR + self.UTC_2_FILE)
                    self.app.ui.checkEarthrotation.setChecked(True)
                    self.app.ui.btn_downloadEarthrotation.setStyleSheet(self.DEFAULT)
                    self.downloadFile(self.SPACESTATIONS, self.TARGET_DIR + self.SPACESTATIONS_FILE)
                    self.app.ui.checkTLE.setChecked(True)
                    self.app.ui.btn_downloadSpacestations.setStyleSheet(self.DEFAULT)
                    self.downloadFile(self.SATBRIGHTEST, self.TARGET_DIR + self.SATBRIGHTEST_FILE)
                    self.app.ui.checkTLE.setChecked(True)
                    self.app.ui.btn_downloadSatbrighest.setStyleSheet(self.DEFAULT)
                    self.downloadFile(self.COMETS, self.TARGET_DIR + self.COMETS_FILE)
                    self.app.ui.btn_downloadComets.setStyleSheet(self.DEFAULT)
                    self.app.ui.checkComets.setChecked(True)
                    self.downloadFile(self.ASTEROIDS_MPC5000, self.TARGET_DIR + self.ASTEROIDS_FILE)
                    self.app.ui.btn_downloadAsteroidsMPC5000.setStyleSheet(self.DEFAULT)
                    self.app.ui.checkAsteroids.setChecked(True)
                    self.app.ui.btn_downloadAll.setStyleSheet(self.DEFAULT)
                elif command == 'UPLOADMOUNT':
                    self.app.ui.btn_uploadMount.setStyleSheet(self.BLUE)
                    self.uploadMount()
                    self.app.ui.btn_uploadMount.setStyleSheet(self.DEFAULT)
                else:
                    pass
            time.sleep(0.3)                                                                                                 # wait for the next cycle
        self.terminate()                                                                                                    # closing the thread at the end

    def __del__(self):                                                                                                      # remove thread
        self.wait()

    def filterFileMPC(self, directory, filename, expression, start, end):
        numberEntry = 0
        with open(directory + filename, 'r') as inFile, open(directory + 'filter.mpc', 'w') as outFile:
            for line in inFile:
                searchExp = expression.split(',')
                for exp in searchExp:
                    if line.find(exp, start, end) != -1:
                        outFile.write(line)
                        numberEntry += 1
        if numberEntry == 0:
            return False
        else:
            self.app.messageQueue.put('Found {0} target(s) in MPC file: {1}!'.format(numberEntry, filename))
            self.logger.info('Found {0} target(s) in MPC file: {1}!'.format(numberEntry, filename))
            return True

    def downloadFile(self, url, filename):
        try:
            u = urllib2.urlopen(url)
            with open(filename, 'wb') as f:
                meta = u.info()
                meta_func = meta.getheaders if hasattr(meta, 'getheaders') else meta.get_all
                meta_length = meta_func("Content-Length")
                file_size = None
                if meta_length:
                    file_size = int(meta_length[0])
                self.app.messageQueue.put('{0}'.format(url))
                file_size_dl = 0
                block_sz = 8192
                while True:
                    buffer = u.read(block_sz)
                    if not buffer:
                        break
                    file_size_dl += len(buffer)
                    f.write(buffer)
            self.app.messageQueue.put('Downloaded {0} Bytes'.format(file_size))
        except Exception as e:
            self.logger.error('Download of {0} failed, error{1}'.format(url, e))
            self.app.messageQueue.put('Download Error {0}'.format(e))
        return

    def uploadMount(self):
        actual_work_dir = ''
        try:
            actual_work_dir = os.getcwd()
            os.chdir(os.path.dirname(self.appInstallPath))
            app = Application(backend='win32')
            app.start(self.appInstallPath + self.appExe)
            # timings.Timings.Slow()
        except application.AppStartError:
            self.logger.error('Failed to start updater, please check!')
            self.app.messageQueue.put('Failed to start updater, please check!')
            os.chdir(actual_work_dir)
            return
        try:
            dialog = timings.WaitUntilPasses(2, 0.2, lambda: findwindows.find_windows(title='GmQCIv2', class_name='#32770')[0])
            winOK = app.window_(handle=dialog)
            winOK['OK'].click()
        except timings.TimeoutError as e:
            self.logger.warning('No invalid floating point windows occurred - moving forward')
        except Exception as e:
            self.logger.error('error{0}'.format(e))
        finally:
            pass
        try:
            win = app['10 micron control box update']                                                                       # link handle
            win['next'].click()                                                                                             # accept next
            win['next'].click()                                                                                             # go upload select page
            ButtonWrapper(win['Control box firmware']).UncheckByClick()                                                       # no firmware updates
        except Exception as e:
            self.logger.error('error{0}'.format(e))
            self.app.messageQueue.put('Error in starting 10micron updater, please check!')
            os.chdir(actual_work_dir)
            return
        ButtonWrapper(win['Orbital parameters of comets']).UncheckByClick()
        ButtonWrapper(win['Orbital parameters of asteroids']).UncheckByClick()
        ButtonWrapper(win['Orbital parameters of satellites']).UncheckByClick()
        ButtonWrapper(win['UTC / Earth rotation data']).UncheckByClick()
        try:
            uploadNecessary = False
            if self.app.ui.checkComets.isChecked():
                ButtonWrapper(win['Orbital parameters of comets']).CheckByClick()
                win['Edit...4'].click()
                popup = app['Comet orbits']
                popup['MPC file'].click()
                filedialog = app[self.OPENDIALOG]
                if self.app.ui.checkFilterMPC.isChecked():
                    if self.filterFileMPC(self.TARGET_DIR, self.COMETS_FILE, self.app.ui.le_filterExpressionMPC.text(), self.COMETS_START, self.COMETS_END):
                        uploadNecessary = True
                    EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + 'filter.mpc')                               # filename box
                else:
                    uploadNecessary = True
                    EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.COMETS_FILE)                           # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                popup['Close'].click()
            else:
                ButtonWrapper(win['Orbital parameters of comets']).UncheckByClick()
            if self.app.ui.checkAsteroids.isChecked():
                ButtonWrapper(win['Orbital parameters of asteroids']).CheckByClick()
                win['Edit...3'].click()
                popup = app['Asteroid orbits']
                popup['MPC file'].click()
                filedialog = app[self.OPENDIALOG]
                if self.app.ui.checkFilterMPC.isChecked():
                    if self.filterFileMPC(self.TARGET_DIR, self.ASTEROIDS_FILE, self.app.ui.le_filterExpressionMPC.text(), self.ASTEROIDS_START, self.ASTEROIDS_END):
                        uploadNecessary = True
                    EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + 'filter.mpc')
                else:
                    uploadNecessary = True
                    EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.ASTEROIDS_FILE)                        # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                popup['Close'].click()
            else:
                ButtonWrapper(win['Orbital parameters of asteroids']).UncheckByClick()
            if self.app.ui.checkTLE.isChecked():
                ButtonWrapper(win['Orbital parameters of satellites']).CheckByClick()
                win['Edit...2'].click()
                popup = app['Satellites orbits']
                popup['Load from file'].click()
                filedialog = app[self.OPENDIALOG]
                EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.SATBRIGHTEST_FILE)                         # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                popup['Close'].click()
                uploadNecessary = True
            else:
                ButtonWrapper(win['Orbital parameters of satellites']).UncheckByClick()
            if self.app.ui.checkTLE.isChecked():
                ButtonWrapper(win['Orbital parameters of satellites']).CheckByClick()
                win['Edit...2'].click()
                popup = app['Satellites orbits']
                popup['Load from file'].click()
                filedialog = app[self.OPENDIALOG]
                EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.SPACESTATIONS_FILE)                        # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                popup['Close'].click()
                uploadNecessary = True
            else:
                ButtonWrapper(win['Orbital parameters of satellites']).UncheckByClick()
            if self.app.ui.checkEarthrotation.isChecked():
                ButtonWrapper(win['UTC / Earth rotation data']).CheckByClick()
                win['Edit...1'].click()
                popup = app['UTC / Earth rotation data']
                popup['Import files...'].click()
                filedialog = app['Open finals data']
                EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.UTC_1_FILE)                                # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                filedialog = app['Open tai-utc.dat']
                EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.UTC_2_FILE)                                # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                fileOK = app['UTC data']
                fileOK['OK'].click()
                uploadNecessary = True
            else:
                ButtonWrapper(win['UTC / Earth rotation data']).UncheckByClick()
        except Exception as e:
            self.logger.error('error{0}'.format(e))
            self.app.messageQueue.put('Error in choosing upload files, please check 10micron updater!')
            os.chdir(actual_work_dir)
            return
        if self.app.mount.mountHandler.sendCommand('GVD') == 'Simulation' or not self.app.mount.mountHandler.connected:     # Upload with real mount which is connected
            self.app.messageQueue.put('Upload only possible with real mount !')
            uploadNecessary = False
        if uploadNecessary:
            try:
                win['next'].click()
                win['next'].click()
                win['Update Now'].click()
            except Exception as e:
                self.logger.error('error{0}'.format(e))
                self.app.messageQueue.put('Error in uploading files, please check 10micron updater!')
                os.chdir(actual_work_dir)
                return
            try:
                dialog = timings.WaitUntilPasses(60, 0.5, lambda: findwindows.find_windows(title='Update completed', class_name='#32770')[0])
                winOK = app.window_(handle=dialog)
                winOK['OK'].click()
            except Exception as e:
                self.logger.error('error{0}'.format(e))
                self.app.messageQueue.put('Error in closing 10micron updater, please check!')
                os.chdir(actual_work_dir)
                return
        else:
            try:
                win['Cancel'].click()
                winOK = app['Exit updater']
                winOK['Yes'].click()
            except Exception as e:
                self.logger.error('error{0}'.format(e))
                self.app.messageQueue.put('Error in closing Updater, please check!')
                os.chdir(actual_work_dir)
                return


if __name__ == "__main__":
    pass



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################

import logging
import math
# import basic stuff
import platform
import threading
import time
# for the sorting
from operator import itemgetter

# import PyQT5 for threading purpose
import PyQt5

if platform.system() == 'Windows':
    # win32com
    import pythoncom
#  mount driver classes
if platform.system() == 'Windows':
    from mount import ascommount
from mount import ipdirect
# astrometry
from astrometry import transform


class Mount(PyQt5.QtCore.QThread):
    logger = logging.getLogger(__name__)                                                                                    # enable logging
    signalMountConnected = PyQt5.QtCore.pyqtSignal([bool], name='mountConnected')                                           # signal for connection status
    signalMountAzAltPointer = PyQt5.QtCore.pyqtSignal([float, float], name='mountAzAltPointer')
    signalMountTrackPreview = PyQt5.QtCore.pyqtSignal(name='mountTrackPreview')

    BLUE = 'background-color: rgb(42, 130, 218)'
    DEFAULT = 'background-color: rgb(32,32,32); color: rgb(192,192,192)'
    BLIND_COMMANDS = ['AP', 'hP', 'PO', 'RT0', 'RT1', 'RT2', 'RT9', 'STOP', 'U2']

    def __init__(self, app):
        super().__init__()                                                                                                  # init of the class parent with super
        self.app = app                                                                                                      # accessing ui object from mount class
        self.data = {}
        if platform.system() == 'Windows':
            self.MountAscom = ascommount.MountAscom(app)                                                                    # set ascom driver class
        self.MountIpDirect = ipdirect.MountIpDirect(app)
        self.mountHandler = self.MountIpDirect
        self.transform = transform.Transform(app)
        self.statusReference = {'0': 'Tracking',
                                '1': 'Stopped after STOP',
                                '2': 'Slewing to park position',
                                '3': 'Unparking',
                                '4': 'Slewing to home position',
                                '5': 'Parked',
                                '6': 'Slewing or going to stop',
                                '7': 'Tracking Off no move',
                                '8': 'Motor low temperature',
                                '9': 'Tracking outside limits',
                                '10': 'Following Satellite',
                                '11': 'User OK Needed',
                                '98': 'Unknown Status',
                                '99': 'Error'
                                }                                                                                           # conversion list Gstat to text
        self.site_lat = '49'                                                                                                # site lat
        self.site_lon = '0'                                                                                                 # site lon
        self.site_height = '0'                                                                                              # site height
        self.sidereal_time = ''                                                                                             # local sidereal time
        self.counter = 0                                                                                                    # counter im main loop
        self.chooserLock = threading.Lock()
        self.initConfig()

    def initConfig(self):
        self.app.ui.pd_chooseMountConnection.addItem('IP Direct Connection')
        if platform.system() == 'Windows':
            self.app.ui.pd_chooseMountConnection.addItem('ASCOM Driver Connection')
        try:
            if platform.system() == 'Windows':
                if 'ASCOMTelescopeDriverName' in self.app.config:
                    self.MountAscom.driverName = self.app.config['ASCOMTelescopeDriverName']
            if 'MountConnection' in self.app.config:
                self.app.ui.pd_chooseMountConnection.setCurrentIndex(int(self.app.config['MountConnection']))
                self.showConfigEntries(int(self.app.config['MountConnection']))
            if 'CheckAutoRefractionCamera' in self.app.config:
                self.app.ui.checkAutoRefractionCamera.setChecked(self.app.config['CheckAutoRefractionCamera'])
            if 'CheckAutoRefractionNotTracking' in self.app.config:
                self.app.ui.checkAutoRefractionNotTracking.setChecked(self.app.config['CheckAutoRefractionNotTracking'])
            if 'MountIP' in self.app.config:
                self.app.ui.le_mountIP.setText(self.app.config['MountIP'])
            if 'MountMAC' in self.app.config:
                self.app.ui.le_mountMAC.setText(self.app.config['MountMAC'])

        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass
        self.app.ui.pd_chooseMountConnection.currentIndexChanged.connect(self.chooseMountConn)

    def storeConfig(self):
        if platform.system() == 'Windows':
            self.app.config['ASCOMTelescopeDriverName'] = self.MountAscom.driverName
        self.app.config['MountConnection'] = self.app.ui.pd_chooseMountConnection.currentIndex()
        self.app.config['CheckAutoRefractionCamera'] = self.app.ui.checkAutoRefractionCamera.isChecked()
        self.app.config['CheckAutoRefractionNotTracking'] = self.app.ui.checkAutoRefractionNotTracking.isChecked()
        self.app.config['MountIP'] = self.app.ui.le_mountIP.text()
        self.app.config['MountMAC'] = self.app.ui.le_mountMAC.text()

    def showConfigEntries(self, index):
        if index == 0:
            self.app.ui.le_mountIP.setVisible(True)
            self.app.ui.le_mountIP.setEnabled(True)
            self.app.ui.le_mountMAC.setVisible(True)
            self.app.ui.le_mountMAC.setEnabled(True)
            self.app.ui.label_mountIP.setVisible(True)
            self.app.ui.label_mountMAC.setVisible(True)

            self.app.ui.btn_setupMountDriver.setVisible(False)
            self.app.ui.btn_setupMountDriver.setEnabled(False)
        elif index == 1:
            self.app.ui.le_mountIP.setVisible(False)
            self.app.ui.le_mountIP.setEnabled(False)
            self.app.ui.le_mountMAC.setVisible(False)
            self.app.ui.le_mountMAC.setEnabled(False)
            self.app.ui.label_mountIP.setVisible(False)
            self.app.ui.label_mountMAC.setVisible(False)

            self.app.ui.btn_setupMountDriver.setVisible(True)
            self.app.ui.btn_setupMountDriver.setEnabled(True)

    def chooseMountConn(self):
        self.chooserLock.acquire()
        if self.mountHandler.connected:
            self.mountHandler.connected = False
            self.mountHandler.disconnect()
        if self.app.ui.pd_chooseMountConnection.currentText().startswith('IP Direct Connection'):
            self.mountHandler = self.MountIpDirect
            self.logger.info('actual driver is IpDirect, IP is: {0}'.format(self.MountIpDirect.mountIP()))
        if self.app.ui.pd_chooseMountConnection.currentText().startswith('ASCOM Driver Connection'):
            self.mountHandler = self.MountAscom
            self.logger.info('actual driver is ASCOM')
        self.showConfigEntries(self.app.ui.pd_chooseMountConnection.currentIndex())
        self.chooserLock.release()

    def run(self):
        if platform.system() == 'Windows':
            pythoncom.CoInitialize()
        self.chooseMountConn()
        self.counter = 0
        while True:
            self.signalMountConnected.emit(self.mountHandler.connected)
            if self.mountHandler.connected:
                if not self.app.mountCommandQueue.empty():
                    command = self.app.mountCommandQueue.get()
                    if command == 'ShowAlignmentModel':
                        num = self.numberModelStars()
                        if num == -1:
                            self.app.messageQueue.put('Show Model not available without real mount')
                        else:
                            self.app.ui.btn_showActualModel.setStyleSheet(self.BLUE)
                            self.showAlignmentModel(self.getAlignmentModel())
                            self.app.ui.btn_showActualModel.setStyleSheet(self.DEFAULT)
                    elif command == 'ClearAlign':
                        if self.numberModelStars() == -1:
                            self.app.messageQueue.put('Clear Align not available without real mount')
                        else:
                            self.mountHandler.sendCommand('delalig')
                    elif command == 'RunTargetRMSAlignment':
                        if self.numberModelStars() == -1:
                            self.app.messageQueue.put('Run Optimize not available without real mount')
                        else:
                            self.app.ui.btn_runTargetRMSAlignment.setStyleSheet(self.BLUE)
                            self.runTargetRMSAlignment()
                            self.app.ui.btn_runTargetRMSAlignment.setStyleSheet(self.DEFAULT)
                    elif command == 'DeleteWorstPoint':
                        if self.numberModelStars() == -1:
                            self.app.messageQueue.put('Delete worst point not available without real mount')
                        else:
                            self.app.ui.btn_deleteWorstPoint.setStyleSheet(self.BLUE)
                            self.deleteWorstPoint()
                            self.app.ui.btn_deleteWorstPoint.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveBackupModel':
                        self.app.ui.btn_saveBackupModel.setStyleSheet(self.BLUE)
                        self.saveBackupModel()
                        self.app.ui.btn_saveBackupModel.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadBackupModel':
                        self.app.ui.btn_loadBackupModel.setStyleSheet(self.BLUE)
                        self.loadBackupModel()
                        self.app.ui.btn_loadBackupModel.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadBaseModel':
                        self.app.ui.btn_loadBaseModel.setStyleSheet(self.BLUE)
                        self.loadBaseModel()
                        self.app.ui.btn_loadBaseModel.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveBaseModel':
                        self.app.ui.btn_saveBaseModel.setStyleSheet(self.BLUE)
                        self.saveBaseModel()
                        self.app.ui.btn_saveBaseModel.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadRefinementModel':
                        self.app.ui.btn_loadRefinementModel.setStyleSheet(self.BLUE)
                        self.loadRefinementModel()
                        self.app.ui.btn_loadRefinementModel.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveRefinementModel':
                        self.app.ui.btn_saveRefinementModel.setStyleSheet(self.BLUE)
                        self.saveRefinementModel()
                        self.app.ui.btn_saveRefinementModel.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadSimpleModel':
                        self.app.ui.btn_loadSimpleModel.setStyleSheet(self.BLUE)
                        self.loadSimpleModel()
                        self.app.ui.btn_loadSimpleModel.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveSimpleModel':
                        self.app.ui.btn_saveSimpleModel.setStyleSheet(self.BLUE)
                        self.saveSimpleModel()
                        self.app.ui.btn_saveSimpleModel.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadDSO1Model':
                        self.app.ui.btn_loadDSO1Model.setStyleSheet(self.BLUE)
                        self.loadDSO1Model()
                        self.app.ui.btn_loadDSO1Model.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveDSO1Model':
                        self.app.ui.btn_saveDSO1Model.setStyleSheet(self.BLUE)
                        self.saveDSO1Model()
                        self.app.ui.btn_saveDSO1Model.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadDSO2Model':
                        self.app.ui.btn_loadDSO2Model.setStyleSheet(self.BLUE)
                        self.loadDSO2Model()
                        self.app.ui.btn_loadDSO2Model.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveDSO2Model':
                        self.app.ui.btn_saveDSO2Model.setStyleSheet(self.BLUE)
                        self.saveDSO2Model()
                        self.app.ui.btn_saveDSO2Model.setStyleSheet(self.DEFAULT)
                    elif command == 'SetRefractionParameter':
                        self.setRefractionParameter()
                    elif command == 'FLIP':
                        self.flipMount()
                    elif command == 'SetupAscomDriver':
                        self.MountAscom.setupDriver()
                    elif command == 'Shutdown':
                        self.mountShutdown()
                    else:
                        self.mountHandler.sendCommand(command)
                    self.app.mountCommandQueue.task_done()
                else:
                    if self.counter == 0:
                        self.getStatusOnce()
                    if self.counter % 2 == 0:
                        self.getStatusFast()
                    if self.counter % 15 == 0:
                        self.getStatusMedium()
                    if self.counter % 150 == 0:
                        self.getStatusSlow()
                time.sleep(0.2)
                self.counter += 1
            else:
                self.mountHandler.connect()
                self.counter = 0
                time.sleep(1)
        self.mountHandler.disconnect()
        if platform.system() == 'Windows':
            pythoncom.CoUninitialize()

    def mountShutdown(self):
        reply = self.mountHandler.sendCommand('shutdown')
        if reply != '1':
            self.logger.error('error: {0}'.format(reply))
            self.app.messageQueue.put('Error in mount shutdown !')
        else:
            self.mountHandler.connected = False
            time.sleep(1)
            self.mountHandler.disconnect()
            self.logger.info('Shutdown mount manually')
            self.app.messageQueue.put('Shutting mount down !')

    def flipMount(self):
        reply = self.mountHandler.sendCommand('FLIP').rstrip('#').strip()
        if reply == '0':
            self.app.messageQueue.put('Flip Mount could not be executed !')
            self.logger.error('error: {0}'.format(reply))

    def numberModelStars(self):
        return int(self.mountHandler.sendCommand('getalst'))

    def getAlignmentModelStatus(self, alignModel):
        if self.data['FW'] < 21500:
            return alignModel
        try:
            reply = self.mountHandler.sendCommand('getain')
            # there should be a reply, format string is "ZZZ.ZZZZ,+AA.AAAA,EE.EEEE,PPP.PP,+OO.OOOO,+aa.aa, +bb.bb,NN,RRRRR.R#"
            if reply:
                # if a single 'E' returns, there is a problem, not further parameter will follow
                if reply != 'E':
                    a1, a2, a3, a4, a5, a6, a7, a8, a9 = reply.split(',')
                    # 'E' could be sent if not calculable or no value available
                    if a1 != 'E':
                        alignModel['azimuth'] = float(a1)
                    else:
                        alignModel['azimuth'] = 0
                    if a2 != 'E':
                        alignModel['altitude'] = float(a2)
                    else:
                        alignModel['altitude'] = 0
                    if a3 != 'E':
                        alignModel['polarError'] = float(a3)
                    else:
                        alignModel['polarError'] = 0
                    if a4 != 'E':
                        alignModel['posAngle'] = float(a4)
                    else:
                        alignModel['posAngle'] = 0
                    if a5 != 'E':
                        alignModel['orthoError'] = float(a5)
                    else:
                        alignModel['orthoError'] = 0
                    if a6 != 'E':
                        alignModel['azimuthKnobs'] = float(a6)
                    else:
                        alignModel['azimuthKnobs'] = 0
                    if a7 != 'E':
                        alignModel['altitudeKnobs'] = float(a7)
                    else:
                        alignModel['altitudeKnobs'] = 0
                    if a8 != 'E':
                        alignModel['terms'] = int(float(a8))
                    else:
                        alignModel['terms'] = 0
                    if a9 != 'E':
                        alignModel['RMS'] = float(a9)
                    else:
                        alignModel['RMS'] = 0
        except Exception as e:
            self.logger.error('receive error getain command: {0}'.format(e))
        finally:
            return alignModel

    def getAlignmentModel(self):
        alignModel = {}
        points = []
        alignModel['azimuth'] = 0.0
        alignModel['altitude'] = 0.0
        alignModel['polarError'] = 0.0
        alignModel['posAngle'] = 0.0
        alignModel['orthoError'] = 0.0
        alignModel['azimuthKnobs'] = 0.0
        alignModel['altitudeKnobs'] = 0.0
        alignModel['terms'] = 0
        alignModel['RMS'] = 0.0
        alignModel['points'] = points
        numberStars = self.numberModelStars()
        alignModel['number'] = numberStars
        if numberStars < 1:
            return alignModel
        alignModel = self.getAlignmentModelStatus(alignModel)
        for i in range(1, numberStars + 1):
            reply = self.mountHandler.sendCommand('getalp{0:d}'.format(i)).split(',')
            ha = reply[0].strip().split('.')[0]
            dec = reply[1].strip().split('.')[0]
            errorRMS = float(reply[2].strip())
            errorAngle = reply[3].strip().rstrip('#')
            dec = dec.replace('*', ':')
            ra_J2000 = self.transform.degStringToDecimal(ha)
            dec_J2000 = self.transform.degStringToDecimal(dec)
            az, alt = self.transform.ra_dec_lst_to_az_alt(ra_J2000, dec_J2000)
            # index should start with 0, but numbering in mount starts with 1
            alignModel['points'].append((i-1, ra_J2000, dec_J2000, az, alt, errorRMS, float(errorAngle)))
        return alignModel

    def retrofitMountData(self, data):
        num = self.numberModelStars()
        if num == len(data):
            alignModel = self.getAlignmentModel()
            self.showAlignmentModel(alignModel)
            for i in range(0, alignModel['number']):
                data[i]['ModelError'] = float(alignModel['points'][i][5])
                data[i]['RaError'] = data[i]['ModelError'] * math.sin(math.radians(alignModel['points'][i][6]))
                data[i]['DecError'] = data[i]['ModelError'] * math.cos(math.radians(alignModel['points'][i][6]))
            self.app.modelLogQueue.put('Mount Model and Model Data synced\n')
        else:
            self.logger.warning('size mount modeling {0} and modeling data {1} do not fit !'.format(num, len(data)))
            self.app.modelLogQueue.put('Mount Model and Model Data could not be synced\n')
            self.app.messageQueue.put('Error- Mount Model and Model Data mismatch!\n')
        return data

    def showAlignmentModel(self, alignModel):
        self.data['ModelStarError'] = 'Downloading data\n'
        for i in range(0, alignModel['number']):
            self.data['ModelStarError'] += '#{0:02d}   AZ: {1:3d}   Alt: {2:3d}   Err: {3:4.1f}\x22   PA: {4:3.0f}\xb0\n'.format(i, int(alignModel['points'][i][3]), int(alignModel['points'][i][4]), alignModel['points'][i][5], alignModel['points'][i][6])
        self.data['ModelStarError'] += 'Downloading finished\n'
        self.data['NumberAlignmentStars'] = alignModel['number']
        self.data['ModelRMSError'] = '{0:3.1f}'.format(alignModel['RMS'])
        self.data['ModelErrorPosAngle'] = '{0:3.1f}'.format(alignModel['posAngle'])
        self.data['ModelPolarError'] = '{0}'.format(self.transform.decimalToDegree(alignModel['polarError']))
        self.data['ModelOrthoError'] = '{0}'.format(self.transform.decimalToDegree(alignModel['orthoError']))
        self.data['ModelErrorAz'] = '{0}'.format(self.transform.decimalToDegree(alignModel['azimuthKnobs']))
        self.data['ModelErrorAlt'] = '{0}'.format(self.transform.decimalToDegree(alignModel['altitudeKnobs']))
        self.data['ModelTerms'] = '{0:2d}'.format(alignModel['terms'])
        if alignModel['azimuthKnobs'] > 0:
            value = '{0:2.2f} left'.format(abs(alignModel['azimuthKnobs']))
        else:
            value = '{0:2.2f} right'.format(abs(alignModel['azimuthKnobs']))
        self.data['ModelKnobTurnAz'] = '{0}'.format(value)
        if alignModel['altitudeKnobs'] > 0:
            value = '{0:2.2f} down'.format(abs(alignModel['altitudeKnobs']))
        else:
            value = '{0:2.2f} up'.format(abs(alignModel['altitudeKnobs']))
        self.data['ModelKnobTurnAlt'] = '{0}'.format(value)
        self.app.showModelErrorPolar()
        return

    def runTargetRMSAlignment(self):
        self.data['ModelStarError'] = 'delete'
        alignModel = self.getAlignmentModel()
        if alignModel['number'] < 4:
            return
        while alignModel['RMS'] > float(self.app.ui.targetRMS.value()) and alignModel['number'] > 3:
            alignModel = self.deleteWorstPointRaw(alignModel)

    def deleteWorstPoint(self):
        alignModel = self.getAlignmentModel()
        self.deleteWorstPointRaw(alignModel)

    def deleteWorstPointRaw(self, alignModel):
        if alignModel['number'] < 4:
            return
        if alignModel['number'] > 3:
            # index 0 is the worst star, index starts with 0
            a = sorted(alignModel['points'], key=itemgetter(5), reverse=True)
            index = a[0][0]
            # numbering in mount starts with 1
            reply = self.mountHandler.sendCommand('delalst{0:d}'.format(index + 1))
            if reply == '1':
                alignModel = self.getAlignmentModel()
                self.app.modeling.modelData.pop(index)
                for i in range(0, alignModel['number']):
                    self.app.modeling.modelData[i]['ModelError'] = float(alignModel['points'][i][5])
                    self.app.modeling.modelData[i]['RaError'] = self.app.modeling.modelData[i]['ModelError'] * math.sin(math.radians(float(alignModel['points'][i][6])))
                    self.app.modeling.modelData[i]['DecError'] = self.app.modeling.modelData[i]['ModelError'] * math.cos(math.radians(float(alignModel['points'][i][6])))
                self.showAlignmentModel(alignModel)
            else:
                self.logger.warning('Point {0} could not be deleted').format(index)
        return alignModel

    def saveModel(self, target):
        num = self.numberModelStars()
        if num == -1:
            self.app.messageQueue.put('Save Model not available without real mount')
            return False
        self.mountHandler.sendCommand('modeldel0' + target)
        reply = self.mountHandler.sendCommand('modelsv0' + target)
        if reply == '1':
            self.app.messageQueue.put('Actual Mount Model saved to file {0}'.format(target))
            return True
        else:
            self.logger.warning('Model {0} could not be saved'.format(target))
            return False

    def loadModel(self, target):
        num = self.numberModelStars()
        if num == -1:
            self.app.messageQueue.put('Load Model not available without real mount')
            return False
        reply = self.mountHandler.sendCommand('modelld0' + target)
        if reply == '1':
            self.app.messageQueue.put('Mount Model loaded from file {0}'.format(target))
            return True
        else:
            self.app.messageQueue.put('There is no modeling named {0} or error while loading'.format(target))
            self.logger.warning('Model {0} could not be loaded'.format(target))
            return False

    def saveBackupModel(self):
        if self.saveModel('BACKUP'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'backup.dat')

    def loadBackupModel(self):
        if self.loadModel('BACKUP'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('backup.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for BACKUP')

    def saveBaseModel(self):
        if self.saveModel('BASE'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'base.dat')
            else:
                self.app.messageQueue.put('No data for BASE')

    def loadBaseModel(self):
        if self.loadModel('BASE'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('base.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for BASE')
        else:
            self.app.messageQueue.put('No Model in Mount called BASE')

    def saveRefinementModel(self):
        if self.saveModel('REFINE'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'refine.dat')
            else:
                self.app.messageQueue.put('No data for REFINE')

    def loadRefinementModel(self):
        if self.loadModel('REFINE'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('refine.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for REFINE')

    def saveActualModel(self):
        if self.saveModel('ACTUAL'):
            if self.app.modeling.modelData:
                if 'Index' in self.app.modeling.modelData[0].keys():
                    self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'actual.dat')
            else:
                self.app.messageQueue.put('No data for ACTUAL')

    def loadActualModel(self):
        if self.loadModel('ACTUAL'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('actual.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for ACTUAL')

    def saveSimpleModel(self):
        if self.saveModel('SIMPLE'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'simple.dat')
            else:
                self.app.messageQueue.put('No data file for SIMPLE')

    def loadSimpleModel(self):
        if self.loadModel('SIMPLE'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('simple.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for SIMPLE')

    def saveDSO1Model(self):
        if self.saveModel('DSO1'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'DSO1.dat')
            else:
                self.app.messageQueue.put('No data file for DSO1')

    def loadDSO1Model(self):
        if self.loadModel('DSO1'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('DSO1.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for DSO1')

    def saveDSO2Model(self):
        if self.saveModel('DSO2'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'DSO2.dat')
            else:
                self.app.messageQueue.put('No data file for DSO2')

    def loadDSO2Model(self):
        if self.loadModel('DSO2'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('dso2.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for DSO2')

    def setRefractionParameter(self):
        if 'Temperature' in self.app.environment.data and 'Pressure' in self.app.environment.data:
            pressure = self.app.environment.data['Pressure']
            temperature = self.app.environment.data['Temperature']
            if self.app.environment.connected == 1:
                if (900.0 < pressure < 1100.0) and (-40.0 < temperature < 50.0):
                    self.mountHandler.sendCommand('SRPRS{0:04.1f}'.format(pressure))
                    if temperature > 0:
                        self.mountHandler.sendCommand('SRTMP+{0:03.1f}'.format(temperature))
                    else:
                        self.mountHandler.sendCommand('SRTMP-{0:3.1f}'.format(-temperature))
                    self.data['RefractionTemperature'] = self.mountHandler.sendCommand('GRTMP')
                    self.data['RefractionPressure'] = self.mountHandler.sendCommand('GRPRS')
                else:
                    self.logger.warning('parameters out of range ! temperature:{0} pressure:{1}'.format(temperature, pressure))

    def getStatusFast(self):
        reply = self.mountHandler.sendCommand('GS')
        if reply:
            self.data['LocalSiderealTime'] = reply.strip('#')
        reply = self.mountHandler.sendCommand('GR')
        if reply:
            self.data['RaJNow'] = self.transform.degStringToDecimal(reply)
        reply = self.mountHandler.sendCommand('GD')
        if reply:
            self.data['DecJNow'] = self.transform.degStringToDecimal(reply)
        reply = self.mountHandler.sendCommand('Ginfo')
        if reply:
            try:
                reply = reply.rstrip('#').strip().split(',')
            except Exception as e:
                self.logger.error('receive error Ginfo command: {0} reply:{1}'.format(e, reply))
            finally:
                pass
            if len(reply) == 8:
                self.data['RaJNow'] = float(reply[0])
                self.data['DecJNow'] = float(reply[1])
                self.data['Pierside'] = reply[2]
                self.data['Az'] = float(reply[3])
                self.data['Alt'] = float(reply[4])
                # needed for 2.14. firmware
                self.data['JulianDate'] = reply[5].rstrip('#')
                self.data['Status'] = int(reply[6])
                self.data['Slewing'] = (reply[7] == '1')
            else:
                self.logger.warning('Ginfo command delivered wrong number of arguments: {0}'.format(reply))
            self.data['RaJ2000'], self.data['DecJ2000'] = self.transform.transformERFA(self.data['RaJNow'], self.data['DecJNow'], 2)
            self.data['TelescopeRA'] = '{0}'.format(self.transform.decimalToDegree(self.data['RaJ2000'], False, False))
            self.data['TelescopeDEC'] = '{0}'.format(self.transform.decimalToDegree(self.data['DecJ2000'], True, False))
            self.data['TelescopeAltitude'] = '{0:03.2f}'.format(self.data['Alt'])
            self.data['TelescopeAzimuth'] = '{0:03.2f}'.format(self.data['Az'])
            self.data['MountStatus'] = '{0}'.format(self.data['Status'])
            self.data['JulianDate'] = '{0}'.format(self.data['JulianDate'][:13])
            if self.data['Pierside'] == str('W'):
                self.data['TelescopePierSide'] = 'WEST'
            else:
                self.data['TelescopePierSide'] = 'EAST'
            self.signalMountAzAltPointer.emit(self.data['Az'], self.data['Alt'])
            self.data['TimeToFlip'] = int(float(self.mountHandler.sendCommand('Gmte')))
            self.data['MeridianLimitTrack'] = int(float(self.mountHandler.sendCommand('Glmt')))
            self.data['MeridianLimitSlew'] = int(float(self.mountHandler.sendCommand('Glms')))
            self.data['TimeToMeridian'] = int(self.data['TimeToFlip'] - self.data['MeridianLimitTrack'] / 360 * 24 * 60)

    def getStatusMedium(self):
        if self.app.ui.checkAutoRefractionNotTracking.isChecked():
            # if there is no tracking, than updating is good
            if self.data['Status'] != 0:
                self.setRefractionParameter()
        if self.app.ui.checkAutoRefractionCamera.isChecked():
            # the same is good if the camera is not in integrating
            if self.app.modeling.imagingHandler.cameraStatus in ['READY - IDLE', 'DOWNLOADING']:
                self.setRefractionParameter()
        self.data['SlewRate'] = self.mountHandler.sendCommand('GMs')
        self.signalMountTrackPreview.emit()

    def getStatusSlow(self):
        self.data['TimeToTrackingLimit'] = self.mountHandler.sendCommand('Gmte')
        self.data['RefractionTemperature'] = self.mountHandler.sendCommand('GRTMP')
        self.data['RefractionPressure'] = self.mountHandler.sendCommand('GRPRS')
        self.data['TelescopeTempDEC'] = self.mountHandler.sendCommand('GTMP1')
        self.data['RefractionStatus'] = self.mountHandler.sendCommand('GREF')
        self.data['UnattendedFlip'] = self.mountHandler.sendCommand('Guaf')
        self.data['MeridianLimitTrack'] = self.mountHandler.sendCommand('Glmt')
        self.data['MeridianLimitSlew'] = self.mountHandler.sendCommand('Glms')
        self.data['DualAxisTracking'] = self.mountHandler.sendCommand('Gdat')
        self.data['CurrentHorizonLimitHigh'] = self.mountHandler.sendCommand('Gh')
        self.data['CurrentHorizonLimitLow'] = self.mountHandler.sendCommand('Go')
        try:
            if self.data['FW'] >= 21500:
                reply = self.mountHandler.sendCommand('GDUTV')
                if reply:
                    valid, expirationDate = reply.split(',')
                    self.data['UTCDataValid'] = valid
                    self.data['UTCDataExpirationDate'] = expirationDate
        except Exception as e:
            self.logger.error('receive error GDUTV command: {0}'.format(e))
        finally:
            pass

    def getStatusOnce(self):
        # Set high precision mode
        self.mountHandler.sendCommand('U2')
        self.site_height = self.mountHandler.sendCommand('Gev')
        lon1 = self.mountHandler.sendCommand('Gg')
        # due to compatibility to LX200 protocol east is negative
        if lon1[0] == '-':
            self.site_lon = lon1.replace('-', '+')
        else:
            self.site_lon = lon1.replace('+', '-')
        self.site_lat = self.mountHandler.sendCommand('Gt')
        self.data['CurrentSiteElevation'] = self.site_height
        self.data['CurrentSiteLongitude'] = lon1
        self.data['CurrentSiteLatitude'] = self.site_lat
        self.data['FirmwareDate'] = self.mountHandler.sendCommand('GVD')
        self.data['FirmwareNumber'] = self.mountHandler.sendCommand('GVN')
        fw = self.data['FirmwareNumber'].split('.')
        if len(fw) == 3:
            self.data['FW'] = int(float(fw[0]) * 10000 + float(fw[1]) * 100 + float(fw[2]))
        else:
            self.data['FW'] = 0
        self.data['FirmwareProductName'] = self.mountHandler.sendCommand('GVP')
        self.data['FirmwareTime'] = self.mountHandler.sendCommand('GVT')
        self.data['HardwareVersion'] = self.mountHandler.sendCommand('GVZ')
        self.logger.info('FW: {0} Number: {1}'.format(self.mountHandler.sendCommand('GVN'), self.data['FW']))
        self.logger.info('Site Lon:{0}'.format(self.site_lon))
        self.logger.info('Site Lat:{0}'.format(self.site_lat))
        self.logger.info('Site Height:{0}'.format(self.site_height))
        self.loadActualModel()
        alignModel = self.getAlignmentModel()
        if not self.app.modeling.modelData and alignModel['RMS'] > 0:
            self.app.messageQueue.put('Model Data will be reconstructed from Mount Data')
            self.app.modeling.modelData = []
            for i in range(0, alignModel['number']):
                self.app.modeling.modelData.append({'ModelError': float(alignModel['points'][i][5]),
                                                    'RaError': float(alignModel['points'][i][5]) * math.sin(math.radians(alignModel['points'][i][6])),
                                                    'DecError': float(alignModel['points'][i][5]) * math.cos(math.radians(alignModel['points'][i][6])),
                                                    'Azimuth': float(alignModel['points'][i][3]),
                                                    'Altitude': float(alignModel['points'][i][4])})
        self.showAlignmentModel(alignModel)



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
from logging import getLogger

# import for the PyQt5 Framework
import PyQt5.QtWidgets
# numerics
import numpy
# matplotlib
from matplotlib import use

from analyse import analysedata
from baseclasses.widget import MwWidget
from gui import analyse_dialog_ui

use('Qt5Agg')
from matplotlib import pyplot as plt
from matplotlib import figure as figure
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas


class ShowAnalyseData(FigureCanvas):

    def __init__(self, parent=None):
        self.fig = figure.Figure(dpi=75, facecolor=(25/256, 25/256, 25/256))
        # self.axes = self.fig.add_axes([0.1, 0.1, 0.8, 0.8])
        self.axes = self.fig.add_subplot(111)
        self.axes.grid(True, color='white')
        self.axes.set_facecolor((32/256, 32/256, 32/256))
        self.axes.tick_params(axis='x', colors='white')
        self.axes.tick_params(axis='y', colors='white')
        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)
        FigureCanvas.updateGeometry(self)


# noinspection PyTypeChecker
def calculateTimeConstant(x_time, y_value):
    timeconstant = 0
    # print(x_time)
    # print(y_value)
    x = 0
    y = 0
    return timeconstant, x, y


# noinspection PyUnresolvedReferences
class AnalyseWindow(MwWidget):
    logger = getLogger(__name__)

    def __init__(self, app):
        super(AnalyseWindow, self).__init__()
        self.app = app
        self.showStatus = False
        self.scaleRA = 10
        self.scaleDEC = 10
        self.scaleError = 10
        self.data = {}
        self.analyse = analysedata.Analyse(self.app)
        self.ui = analyse_dialog_ui.Ui_AnalyseDialog()
        self.ui.setupUi(self)
        self.initUI()
        self.ui.scalePlotDEC.valueChanged.connect(self.changedDECScale)
        self.ui.scalePlotRA.valueChanged.connect(self.changedRAScale)
        self.ui.scalePlotError.valueChanged.connect(self.changedPlotError)
        self.ui.btn_selectDecError.clicked.connect(self.showDecError)
        self.ui.btn_selectDecErrorDeviation.clicked.connect(self.showDecErrorDeviation)
        self.ui.btn_selectRaError.clicked.connect(self.showRaError)
        self.ui.btn_selectRaErrorDeviation.clicked.connect(self.showRaErrorDeviation)
        self.ui.btn_selectDecErrorAltitude.clicked.connect(self.showDecErrorAltitude)
        self.ui.btn_selectRaErrorAltitude.clicked.connect(self.showRaErrorAltitude)
        self.ui.btn_selectDecErrorAzimuth.clicked.connect(self.showDecErrorAzimuth)
        self.ui.btn_selectRaErrorAzimuth.clicked.connect(self.showRaErrorAzimuth)
        self.ui.btn_selectModelPointPolar.clicked.connect(self.showModelPointPolar)
        self.ui.btn_selectModelPointErrorPolar.clicked.connect(self.showModelPointErrorPolar)
        helper = PyQt5.QtWidgets.QVBoxLayout(self.ui.plot)
        self.plotWidget = ShowAnalyseData(self.ui.plot)
        helper.addWidget(self.plotWidget)
        self.initConfig()
        # self.show()
        self.setVisible(False)

    def initConfig(self):
        try:
            if 'ScalePlotRA' in self.app.config:
                self.ui.scalePlotRA.setValue(self.app.config['ScalePlotRA'])
            if 'ScalePlotDEC' in self.app.config:
                self.ui.scalePlotDEC.setValue(self.app.config['ScalePlotDEC'])
            if 'ScalePlotError' in self.app.config:
                self.ui.scalePlotError.setValue(self.app.config['ScalePlotError'])
            if 'AnalysePopupWindowPositionX' in self.app.config:
                self.move(self.app.config['AnalysePopupWindowPositionX'], self.app.config['AnalysePopupWindowPositionY'])
            if 'AnalysePopupWindowShowStatus' in self.app.config:
                self.showStatus = self.app.config['AnalysePopupWindowShowStatus']
        except Exception as e:
            self.logger.error('Item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass

    def storeConfig(self):
        self.app.config['AnalysePopupWindowPositionX'] = self.pos().x()
        self.app.config['AnalysePopupWindowPositionY'] = self.pos().y()
        self.app.config['AnalysePopupWindowShowStatus'] = self.showStatus
        self.app.config['ScalePlotRA'] = self.ui.scalePlotRA.value()
        self.app.config['ScalePlotDEC'] = self.ui.scalePlotDEC.value()
        self.app.config['ScalePlotError'] = self.ui.scalePlotError.value()

    def showAnalyseWindow(self):
        self.getData()
        self.setWindowTitle('Analyse:    ' + self.app.ui.le_analyseFileName.text())
        self.showDecError()
        self.showStatus = True
        self.setVisible(True)
        self.show()

    def changedDECScale(self):
        if self.getData():
            self.showDecError()

    def changedRAScale(self):
        if self.getData():
            self.showRaError()

    def changedPlotError(self):
        if self.getData():
            self.showModelPointErrorPolar()

    def getData(self):
        filename = self.app.ui.le_analyseFileName.text()
        if filename == '':
            return False
        self.scaleRA = self.ui.scalePlotRA.value()
        self.scaleDEC = self.ui.scalePlotDEC.value()
        self.scaleError = self.ui.scalePlotError.value()
        self.data = self.analyse.loadData(filename)
        return True

    def setFigure(self, projection=None):
        self.plotWidget.fig.clf()
        self.plotWidget.axes = self.plotWidget.fig.add_subplot(1, 1, 1, projection=projection)
        self.plotWidget.axes.grid(True, color='gray')
        self.plotWidget.axes.set_facecolor((32/256, 32/256, 32/256))
        self.plotWidget.axes.tick_params(axis='x', colors='white')
        self.plotWidget.axes.tick_params(axis='y', colors='white')

    def showDecError(self):
        if len(self.data) > 0:
            self.data = self.analyse.prepareData(self.data, self.scaleRA, self.scaleDEC)
        else:
            return
        self.setFigure()
        self.plotWidget.axes.set_xlabel('Number of Model Point', color='white')                                             # x axis
        self.plotWidget.axes.set_ylabel('DEC Error (arcsec)', color='white')                                                # y axis
        self.plotWidget.axes.set_title('DEC Error over Modeling\n ', color='white')                                         # title
        self.plotWidget.axes.set_xlim(0, len(self.data['Index']))
        self.plotWidget.axes.set_ylim(-self.scaleDEC, self.scaleDEC)                                                        # defining the scaling of the plot
        self.plotWidget.axes.plot(self.data['Index'], self.data['DecError'], color='black')                                 # Basic Data
        colors = numpy.asarray(['blue' if x > 180 else 'green' for x in self.data['Azimuth']])
        self.plotWidget.axes.scatter(self.data['Index'], self.data['DecError'], c=colors, s=50)
        self.plotWidget.draw()                                                                                              # put the plot in the widget

    def showDecErrorDeviation(self):
        if len(self.data) > 0:
            self.data = self.analyse.prepareData(self.data, self.scaleRA, self.scaleDEC)
        else:
            return
        # timeconstant, x, y = calculateTimeConstant(self.data['sidereal_time'], self.data['decError'])
        self.setFigure()
        self.plotWidget.axes.set_xlabel('Number of Model Point', color='white')                                             # x axis
        self.plotWidget.axes.set_ylabel('DEC Error(arcsec)', color='white')                                                 # y axis
        self.plotWidget.axes.set_title('DEC Error referenced to 0 over Modeling\n ', color='white')                         # title
        self.plotWidget.axes.set_xlim(0, len(self.data['Index']))
        self.plotWidget.axes.set_ylim(-self.scaleDEC, self.scaleDEC)                                                        # defining the scaling of the plot
        decErrorDeviation = numpy.asarray(self.data['DecError'])
        self.plotWidget.axes.plot(self.data['Index'], decErrorDeviation - decErrorDeviation[0], color='black')              # Basic Data
        colors = numpy.asarray(['blue' if x > 180 else 'green' for x in self.data['Azimuth']])
        self.plotWidget.axes.scatter(self.data['Index'], decErrorDeviation - decErrorDeviation[0], c=colors, s=50)
        self.plotWidget.draw()                                                                                              # put the plot in the widget

    def showRaError(self):
        if len(self.data) > 0:
            self.data = self.analyse.prepareData(self.data, self.scaleRA, self.scaleDEC)
        else:
            return
        self.setFigure()
        self.plotWidget.axes.set_xlabel('Number of Model Point', color='white')
        self.plotWidget.axes.set_ylabel('RA Error (arcsec)', color='white')
        self.plotWidget.axes.set_title('RA Error over Modeling\n ', color='white')
        self.plotWidget.axes.set_xlim(0, len(self.data['Index']))
        self.plotWidget.axes.set_ylim(-self.scaleRA, self.scaleRA)
        self.plotWidget.axes.plot(self.data['Index'], self.data['RaError'], color='black')                                   # Basic Data
        colors = numpy.asarray(['blue' if x > 180 else 'green' for x in self.data['Azimuth']])
        self.plotWidget.axes.scatter(self.data['Index'], self.data['RaError'], c=colors, s=50)
        self.plotWidget.draw()

    def showRaErrorDeviation(self):
        if len(self.data) > 0:
            self.data = self.analyse.prepareData(self.data, self.scaleRA, self.scaleDEC)
        else:
            return
        self.setFigure()
        self.plotWidget.axes.set_xlabel('Number of Model Point', color='white')
        self.plotWidget.axes.set_ylabel('RA Error', color='white')
        self.plotWidget.axes.set_title('RA Error referenced to 0 over Modeling\n ', color='white')
        self.plotWidget.axes.axis([0, len(self.data['Index']), -self.scaleRA, self.scaleRA])
        raErrorDeviation = numpy.asarray(self.data['RaError'])
        self.plotWidget.axes.plot(self.data['Index'], raErrorDeviation - raErrorDeviation[0], color='black')                 # Basic Data
        colors = numpy.asarray(['blue' if x > 180 else 'green' for x in self.data['Azimuth']])
        self.plotWidget.axes.scatter(self.data['Index'], raErrorDeviation - raErrorDeviation[0], c=colors, s=50)
        self.plotWidget.draw()

    def showDecErrorAltitude(self):
        if len(self.data) > 0:
            self.data = self.analyse.prepareData(self.data, self.scaleRA, self.scaleDEC)
        else:
            return
        self.setFigure()
        self.plotWidget.axes.set_xlabel('Altitude', color='white')
        self.plotWidget.axes.set_ylabel('DEC Error (arcsec)', color='white')
        self.plotWidget.axes.set_title('DEC Error over Altitude\n ', color='white')
        self.plotWidget.axes.set_xlim(0, 90)
        self.plotWidget.axes.set_ylim(-self.scaleDEC, self.scaleDEC)
        self.plotWidget.axes.plot(self.data['Altitude'], self.data['DecError'], color='black')
        colors = numpy.asarray(['blue' if x > 180 else 'green' for x in self.data['Azimuth']])
        self.plotWidget.axes.scatter(self.data['Altitude'], self.data['DecError'], c=colors, s=50)
        self.plotWidget.draw()

    def showRaErrorAltitude(self):
        if len(self.data) > 0:
            self.data = self.analyse.prepareData(self.data, self.scaleRA, self.scaleDEC)
        else:
            return
        self.setFigure()
        self.plotWidget.axes.set_xlabel('Altitude', color='white')
        self.plotWidget.axes.set_ylabel('RA Error (arcsec)', color='white')
        self.plotWidget.axes.set_title('RA Error over Altitude\n ', color='white')
        self.plotWidget.axes.set_xlim(0, 90)
        self.plotWidget.axes.set_ylim(-self.scaleRA, self.scaleRA)
        self.plotWidget.axes.plot(self.data['Altitude'], self.data['RaError'], color='black')
        colors = numpy.asarray(['blue' if x > 180 else 'green' for x in self.data['Azimuth']])
        self.plotWidget.axes.scatter(self.data['Altitude'], self.data['RaError'], c=colors, s=50)
        self.plotWidget.draw()

    def showDecErrorAzimuth(self):
        if len(self.data) > 0:
            self.data = self.analyse.prepareData(self.data, self.scaleRA, self.scaleDEC)
        else:
            return
        self.setFigure()
        self.plotWidget.axes.set_xlabel('Altitude', color='white')
        self.plotWidget.axes.set_ylabel('DEC Error (arcsec)', color='white')
        self.plotWidget.axes.set_title('DEC Error over Azimuth\n ', color='white')
        self.plotWidget.axes.set_xlim(0, 360)
        self.plotWidget.axes.set_ylim(-self.scaleDEC, self.scaleDEC)
        self.plotWidget.axes.plot(self.data['Azimuth'], self.data['DecError'], color='black')
        colors = numpy.asarray(['blue' if x > 180 else 'green' for x in self.data['Azimuth']])
        self.plotWidget.axes.scatter(self.data['Azimuth'], self.data['DecError'], c=colors, s=50)
        self.plotWidget.draw()

    def showRaErrorAzimuth(self):
        if len(self.data) > 0:
            self.data = self.analyse.prepareData(self.data, self.scaleRA, self.scaleDEC)
        else:
            return
        self.setFigure()
        self.plotWidget.axes.set_xlabel('Altitude', color='white')
        self.plotWidget.axes.set_ylabel('RA Error (arcsec)', color='white')
        self.plotWidget.axes.set_title('RA Error over Azimuth\n ', color='white')
        self.plotWidget.axes.set_xlim(0, 360)
        self.plotWidget.axes.set_ylim(-self.scaleRA, self.scaleRA)
        self.plotWidget.axes.plot(self.data['Azimuth'], self.data['RaError'], color='black')
        colors = numpy.asarray(['blue' if x > 180 else 'green' for x in self.data['Azimuth']])
        self.plotWidget.axes.scatter(self.data['Azimuth'], self.data['RaError'], c=colors, s=50)
        self.plotWidget.draw()

    def showModelPointPolar(self):
        self.setFigure('polar')
        self.plotWidget.axes.set_theta_zero_location('N')
        self.plotWidget.axes.set_theta_direction(-1)
        self.plotWidget.axes.set_yticks(range(0, 90, 10))
        yLabel = ['', '80', '', '60', '', '40', '', '20', '', '0']
        self.plotWidget.axes.set_yticklabels(yLabel, color='white')
        self.plotWidget.axes.set_title('Model Points\n ', color='white')
        azimuth = numpy.asarray(self.data['Azimuth'])
        altitude = numpy.asarray(self.data['Altitude'])
        colors = numpy.asarray(['blue' if x > 180 else 'green' for x in self.data['Azimuth']])
        self.plotWidget.axes.plot(azimuth / 180.0 * 3.141593, 90 - altitude, color='black')
        self.plotWidget.axes.scatter(azimuth / 180.0 * 3.141593, 90 - altitude, c=colors, s=50)
        self.plotWidget.axes.set_rmax(90)
        self.plotWidget.axes.set_rmin(0)
        self.plotWidget.draw()

    def showModelPointErrorPolar(self):
        self.setFigure('polar')
        self.plotWidget.axes.set_theta_zero_location('N')
        self.plotWidget.axes.set_theta_direction(-1)
        self.plotWidget.axes.set_yticks(range(0, 90, 10))
        yLabel = ['', '80', '', '60', '', '40', '', '20', '', '0']
        self.plotWidget.axes.set_yticklabels(yLabel, color='white')
        self.plotWidget.axes.set_title('Model Points Error\n ', color='white')
        azimuth = numpy.asarray(self.data['Azimuth'])
        altitude = numpy.asarray(self.data['Altitude'])
        self.plotWidget.axes.plot(azimuth / 180.0 * 3.141593, 90 - altitude, color='black')
        cm = plt.cm.get_cmap('RdYlGn_r')
        colors = numpy.asarray(self.data['ModelError'])
        scaleError = int(max(colors) / 4 + 1) * 4
        area = colors * 100 / self.scaleError + 20
        theta = azimuth / 180.0 * 3.141593
        r = 90 - altitude
        scatter = self.plotWidget.axes.scatter(theta, r, c=colors, vmin=0, vmax=scaleError, s=area, cmap=cm)
        scatter.set_alpha(0.75)
        colorbar = self.plotWidget.fig.colorbar(scatter)
        colorbar.set_label('Error [arcsec]', color='white')
        plt.setp(plt.getp(colorbar.ax.axes, 'yticklabels'), color='white')
        self.plotWidget.axes.set_rmax(90)
        self.plotWidget.axes.set_rmin(0)
        self.plotWidget.draw()



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
import copy
import logging
import os
import platform
import time
# threading
import threading
# queuing
from queue import Queue
# PyQt5
import PyQt5
# for data storing
from analyse.analysedata import Analyse
# cameras
from camera import none
from camera import indicamera
if platform.system() == 'Windows':
    from camera import maximdl
    from camera import sgpro
    import pythoncom
if platform.system() == 'Windows' or platform.system() == 'Darwin':
    from camera import theskyx
# modelPoints
from modeling import modelPoints
# workers
from modeling import modelWorker


class Modeling(PyQt5.QtCore.QThread):
    logger = logging.getLogger(__name__)
    signalModelConnected = PyQt5.QtCore.pyqtSignal(int, name='ModelConnected')
    signalModelRedraw = PyQt5.QtCore.pyqtSignal(bool, name='ModelRedrawPoints')

    BLUE = 'background-color: rgb(42, 130, 218)'
    RED = 'background-color: red;'
    DEFAULT = 'background-color: rgb(32,32,32); color: rgb(192,192,192)'
    REF_PICTURE = '/model001.fit'
    IMAGEDIR = os.getcwd().replace('\\', '/') + '/images'
    CAPTUREFILE = 'modeling'
    CYCLESTATUSFAST = 1000

    def __init__(self, app):
        super(PyQt5.QtCore.QThread, self).__init__()
        # make main sources available
        self.app = app
        self.isRunning = True
        # make windows imaging applications available
        if platform.system() == 'Windows':
            self.SGPro = sgpro.SGPro(self.app)
            self.MaximDL = maximdl.MaximDLCamera(self.app)
        if platform.system() == 'Windows' or platform.system() == 'Darwin':
            self.TheSkyX = theskyx.TheSkyX(self.app)
        # make non windows applications available
        self.NoneCam = none.NoneCamera(self.app)
        self.INDICamera = indicamera.INDICamera(self.app)
        # select default application
        self.imagingHandler = self.NoneCam
        # assign support classes
        self.analyse = Analyse(self.app)
        self.transform = self.app.mount.transform
        self.modelPoints = modelPoints.ModelPoints(self.transform)
        self.modelWorker = modelWorker.ModelWorker(self.app)
        # counter for thread timing
        self.counter = 0
        self.chooserLock = threading.Lock()
        # finally initialize the class configuration
        self.cancel = False
        self.modelRun = False
        self.modelAnalyseData = []
        self.modelData = {}
        self.counter = 0
        # setting the config up
        self.initConfig()

    def initConfig(self):
        if self.NoneCam.appAvailable:
            self.app.ui.pd_chooseImagingApp.addItem('No Application')
        if self.INDICamera.appAvailable:
            self.app.ui.pd_chooseImagingApp.addItem('INDI Camera')
        if platform.system() == 'Windows':
            if self.SGPro.appAvailable:
                self.app.ui.pd_chooseImagingApp.addItem('SGPro - ' + self.SGPro.appName)
            if self.MaximDL.appAvailable:
                self.app.ui.pd_chooseImagingApp.addItem('MaximDL - ' + self.MaximDL.appName)
        if platform.system() == 'Windows' or platform.system() == 'Darwin':
            if self.TheSkyX.appAvailable:
                self.app.ui.pd_chooseImagingApp.addItem('TheSkyX - ' + self.TheSkyX.appName)
        try:
            if 'ImagingApplication' in self.app.config:
                self.app.ui.pd_chooseImagingApp.setCurrentIndex(int(self.app.config['ImagingApplication']))
            if 'CheckSortPoints' in self.app.config:
                self.app.ui.checkSortPoints.setChecked(self.app.config['CheckSortPoints'])
            if 'CheckDeletePointsHorizonMask' in self.app.config:
                self.app.ui.checkDeletePointsHorizonMask.setChecked(self.app.config['CheckDeletePointsHorizonMask'])
            if 'CheckSimulation' in self.app.config:
                self.app.ui.checkSimulation.setChecked(self.app.config['CheckSimulation'])
        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass

    def storeConfig(self):
        self.app.config['ImagingApplication'] = self.app.ui.pd_chooseImagingApp.currentIndex()
        self.app.config['CheckSortPoints'] = self.app.ui.checkSortPoints.isChecked()
        self.app.config['CheckDeletePointsHorizonMask'] = self.app.ui.checkDeletePointsHorizonMask.isChecked()
        self.app.config['CheckSimulation'] = self.app.ui.checkSimulation.isChecked()

    def chooseImagingApp(self):
        self.chooserLock.acquire()
        if self.imagingHandler.cameraConnected:
            self.imagingHandler.disconnectCamera()
        if self.app.ui.pd_chooseImagingApp.currentText().startswith('No Application'):
            self.imagingHandler = self.NoneCam
            self.logger.info('actual camera / plate solver is None')
        elif self.app.ui.pd_chooseImagingApp.currentText().startswith('INDI Camera'):
            self.imagingHandler = self.INDICamera
            self.logger.info('actual camera / plate solver is INDI Camera')
        elif self.app.ui.pd_chooseImagingApp.currentText().startswith('SGPro'):
            self.imagingHandler = self.SGPro
            self.logger.info('actual camera / plate solver is SGPro')
        elif self.app.ui.pd_chooseImagingApp.currentText().startswith('TheSkyX'):
            self.imagingHandler = self.TheSkyX
            self.logger.info('actual camera / plate solver is TheSkyX')
        elif self.app.ui.pd_chooseImagingApp.currentText().startswith('MaximDL'):
            self.imagingHandler = self.MaximDL
            self.logger.info('actual camera / plate solver is MaximDL')
        self.imagingHandler.checkAppStatus()
        self.imagingHandler.connectCamera()
        self.chooserLock.release()

    def run(self):
        if platform.system() == 'Windows':
            pythoncom.CoInitialize()
        # run it first, to set all imaging applications up
        self.chooseImagingApp()
        # connect change in imaging app to the subroutine of setting it up
        self.app.ui.pd_chooseImagingApp.currentIndexChanged.connect(self.chooseImagingApp)
        # start first time the loop for status updates
        self.getStatusFast()
        while self.isRunning:
            if not self.app.modelCommandQueue.empty():
                command = self.app.modelCommandQueue.get()
            else:
                command = ''
            if self.app.mount.mountHandler.connected:
                if self.imagingHandler.cameraConnected:
                    if command == 'RunBaseModel':
                        self.app.imageWindow.disableExposures()
                        self.app.ui.btn_runBaseModel.setStyleSheet(self.BLUE)
                        self.modelWorker.runBaseModel()
                        self.app.ui.btn_runBaseModel.setStyleSheet(self.DEFAULT)
                        self.app.ui.btn_cancelModel.setStyleSheet(self.DEFAULT)
                        self.app.imageWindow.enableExposures()
                    elif command == 'RunRefinementModel':
                        self.app.imageWindow.disableExposures()
                        self.app.ui.btn_runRefinementModel.setStyleSheet(self.BLUE)
                        self.modelWorker.runRefinementModel()
                        self.app.ui.btn_runRefinementModel.setStyleSheet(self.DEFAULT)
                        self.app.ui.btn_cancelModel.setStyleSheet(self.DEFAULT)
                        self.app.imageWindow.enableExposures()
                    elif command == 'RunBoostModel':
                        self.app.imageWindow.disableExposures()
                        self.app.ui.btn_runBoostModel.setStyleSheet(self.BLUE)
                        simulation = self.app.ui.checkSimulation.isChecked()
                        self.runBoostModeling(simulation)
                        self.app.ui.btn_runBoostModel.setStyleSheet(self.DEFAULT)
                        self.app.ui.btn_cancelModel.setStyleSheet(self.DEFAULT)
                        self.app.imageWindow.enableExposures()
                    elif command == 'PlateSolveSync':
                        self.app.imageWindow.disableExposures()
                        self.app.ui.btn_plateSolveSync.setStyleSheet(self.BLUE)
                        simulation = self.app.ui.checkSimulation.isChecked()
                        self.modelWorker.plateSolveSync(simulation)
                        self.app.ui.btn_plateSolveSync.setStyleSheet(self.DEFAULT)
                        self.app.imageWindow.enableExposures()
                    elif command == 'RunBatchModel':
                        self.app.ui.btn_runBatchModel.setStyleSheet(self.BLUE)
                        self.modelWorker.runBatchModel()
                        self.app.ui.btn_runBatchModel.setStyleSheet(self.DEFAULT)
                    elif command == 'RunCheckModel':
                        self.app.imageWindow.disableExposures()
                        self.app.ui.btn_runCheckModel.setStyleSheet(self.BLUE)
                        num = self.app.mount.numberModelStars()
                        if num > 2:
                            self.modelWorker.runCheckModel()
                        else:
                            self.app.modelLogQueue.put('Run Analyse stopped, not BASE modeling available !\n')
                            self.app.messageQueue.put('Run Analyse stopped, not BASE modeling available !\n')
                        self.app.ui.btn_runCheckModel.setStyleSheet(self.DEFAULT)
                        self.app.ui.btn_cancelModel.setStyleSheet(self.DEFAULT)
                        self.app.imageWindow.enableExposures()
                    elif command == 'RunAllModel':
                        self.app.imageWindow.disableExposures()
                        self.app.ui.btn_runAllModel.setStyleSheet(self.BLUE)
                        self.modelWorker.runAllModel()
                        self.app.ui.btn_runAllModel.setStyleSheet(self.DEFAULT)
                        self.app.ui.btn_cancelModel.setStyleSheet(self.DEFAULT)
                        self.app.imageWindow.enableExposures()
                    elif command == 'RunTimeChangeModel':
                        self.app.imageWindow.disableExposures()
                        self.app.ui.btn_runTimeChangeModel.setStyleSheet(self.BLUE)
                        self.modelWorker.runTimeChangeModel()
                        self.app.ui.btn_runTimeChangeModel.setStyleSheet(self.DEFAULT)
                        self.app.ui.btn_cancelAnalyseModel.setStyleSheet(self.DEFAULT)
                        self.app.imageWindow.enableExposures()
                    elif command == 'RunHystereseModel':
                        self.app.imageWindow.disableExposures()
                        self.app.ui.btn_runHystereseModel.setStyleSheet(self.BLUE)
                        self.modelWorker.runHystereseModel()
                        self.app.ui.btn_runHystereseModel.setStyleSheet(self.DEFAULT)
                        self.app.ui.btn_cancelAnalyseModel.setStyleSheet(self.DEFAULT)
                        self.app.imageWindow.enableExposures()
                    elif command == 'ClearAlignmentModel':
                        self.app.ui.btn_clearAlignmentModel.setStyleSheet(self.BLUE)
                        self.app.modelLogQueue.put('Clearing alignment modeling - taking 4 seconds.\n')
                        self.modelWorker.clearAlignmentModel()
                        self.app.modelLogQueue.put('Model cleared!\n')
                        self.app.ui.btn_clearAlignmentModel.setStyleSheet(self.DEFAULT)
                if command == 'GenerateDSOPoints':
                    self.app.ui.btn_generateDSOPoints.setStyleSheet(self.BLUE)
                    self.modelPoints.generateDSOPoints(int(float(self.app.ui.numberHoursDSO.value())),
                                                       int(float(self.app.ui.numberPointsDSO.value())),
                                                       int(float(self.app.ui.numberHoursPreview.value())),
                                                       copy.copy(self.app.mount.data['RaJNow']),
                                                       copy.copy(self.app.mount.data['DecJNow']))
                    if self.app.ui.checkSortPoints.isChecked():
                        self.modelPoints.sortPoints('refinement')
                    if self.app.ui.checkDeletePointsHorizonMask.isChecked():
                        self.modelPoints.deleteBelowHorizonLine()
                    self.signalModelRedraw.emit(True)
                    self.app.ui.btn_generateDSOPoints.setStyleSheet(self.DEFAULT)
                elif command == 'GenerateDensePoints':
                    self.app.ui.btn_generateDensePoints.setStyleSheet(self.BLUE)
                    self.modelPoints.generateDensePoints()
                    if self.app.ui.checkSortPoints.isChecked():
                        self.modelPoints.sortPoints('refinement')
                    if self.app.ui.checkDeletePointsHorizonMask.isChecked():
                        self.modelPoints.deleteBelowHorizonLine()
                    self.signalModelRedraw.emit(True)
                    self.app.ui.btn_generateDensePoints.setStyleSheet(self.DEFAULT)
                elif command == 'GenerateNormalPoints':
                    self.app.ui.btn_generateNormalPoints.setStyleSheet(self.BLUE)
                    self.modelPoints.generateNormalPoints()
                    if self.app.ui.checkSortPoints.isChecked():
                        self.modelPoints.sortPoints('refinement')
                    if self.app.ui.checkDeletePointsHorizonMask.isChecked():
                        self.modelPoints.deleteBelowHorizonLine()
                    self.signalModelRedraw.emit(True)
                    self.app.ui.btn_generateNormalPoints.setStyleSheet(self.DEFAULT)
                else:
                    pass
            if command == 'LoadBasePoints':
                self.modelPoints.loadBasePoints(self.app.ui.le_modelPointsFileName.text())
                self.signalModelRedraw.emit(True)
            elif command == 'LoadRefinementPoints':
                self.modelPoints.loadRefinementPoints(self.app.ui.le_modelPointsFileName.text())
                if self.app.ui.checkSortPoints.isChecked():
                    self.modelPoints.sortPoints('refinement')
                if self.app.ui.checkDeletePointsHorizonMask.isChecked():
                    self.modelPoints.deleteBelowHorizonLine()
                self.signalModelRedraw.emit(True)
            elif command == 'GenerateGridPoints':
                self.app.ui.btn_generateGridPoints.setStyleSheet(self.BLUE)
                self.modelPoints.generateGridPoints(int(float(self.app.ui.numberGridPointsRow.value())),
                                                    int(float(self.app.ui.numberGridPointsCol.value())),
                                                    int(float(self.app.ui.altitudeMin.value())),
                                                    int(float(self.app.ui.altitudeMax.value())))
                if self.app.ui.checkSortPoints.isChecked():
                    self.modelPoints.sortPoints('refinement')
                if self.app.ui.checkDeletePointsHorizonMask.isChecked():
                    self.modelPoints.deleteBelowHorizonLine()
                self.signalModelRedraw.emit(True)
                self.app.ui.btn_generateGridPoints.setStyleSheet(self.DEFAULT)
            elif command == 'GenerateBasePoints':
                self.modelPoints.generateBasePoints(float(self.app.ui.azimuthBase.value()),
                                                    float(self.app.ui.altitudeBase.value()))
                self.signalModelRedraw.emit(True)
            elif command == 'DeletePoints':
                self.modelPoints.deletePoints()
                self.signalModelRedraw.emit(True)
            self.counter += 1
            if self.counter % 5 == 0:
                self.getStatusFast()
            time.sleep(0.2)
            PyQt5.QtWidgets.QApplication.processEvents()

    def stop(self):
        if platform.system() == 'Windows':
            pythoncom.CoUninitialize()
        self.isRunning = False

    def cancelModeling(self):
        if self.modelRun:
            self.app.ui.btn_cancelModel.setStyleSheet(self.RED)
            self.logger.info('User canceled modeling with cancel any model run')
            self.cancel = True

    def cancelAnalyseModeling(self):
        if self.modelRun:
            self.app.ui.btn_cancelAnalyseModel.setStyleSheet(self.RED)
            self.logger.info('User canceled modeling with cancel analyse run')
            self.cancel = True

    def getStatusFast(self):
        self.imagingHandler.checkAppStatus()
        self.imagingHandler.getCameraStatus()
        self.signalModelConnected.emit(1)
        if self.imagingHandler.appRunning:
            self.signalModelConnected.emit(2)
        if self.imagingHandler.cameraConnected:
            self.signalModelConnected.emit(3)

    def runBoostModeling(self, simulation):
        # loading test data
        for i in range(1, 10):
            self.workerSlewpoint.queuePoint.put(i)
        # start process
        self.workerSlewpoint.signalSlewing.emit()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'popup_dialog_ui.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PopupDialog(object):
    def setupUi(self, PopupDialog):
        PopupDialog.setObjectName("PopupDialog")
        PopupDialog.resize(400, 511)
        self.windowTitle = QtWidgets.QLabel(PopupDialog)
        self.windowTitle.setGeometry(QtCore.QRect(0, 0, 401, 31))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.windowTitle.setFont(font)
        self.windowTitle.setAutoFillBackground(True)
        self.windowTitle.setAlignment(QtCore.Qt.AlignCenter)
        self.windowTitle.setObjectName("windowTitle")
        self.widgetPlot = QtWidgets.QWidget(PopupDialog)
        self.widgetPlot.setGeometry(QtCore.QRect(-1, 100, 401, 411))
        self.widgetPlot.setObjectName("widgetPlot")

        self.retranslateUi(PopupDialog)
        QtCore.QMetaObject.connectSlotsByName(PopupDialog)

    def retranslateUi(self, PopupDialog):
        _translate = QtCore.QCoreApplication.translate
        PopupDialog.setWindowTitle(_translate("PopupDialog", "Form"))
        self.windowTitle.setText(_translate("PopupDialog", "Popup"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'image_dialog_ui.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ImageDialog(object):
    def setupUi(self, ImageDialog):
        ImageDialog.setObjectName("ImageDialog")
        ImageDialog.resize(791, 639)
        font = QtGui.QFont()
        font.setFamily("Arial")
        ImageDialog.setFont(font)
        self.image = QtWidgets.QWidget(ImageDialog)
        self.image.setGeometry(QtCore.QRect(0, 105, 791, 531))
        self.image.setAutoFillBackground(True)
        self.image.setObjectName("image")
        self.btn_expose = QtWidgets.QPushButton(ImageDialog)
        self.btn_expose.setGeometry(QtCore.QRect(10, 10, 81, 81))
        self.btn_expose.setObjectName("btn_expose")
        self.btn_crosshair = QtWidgets.QPushButton(ImageDialog)
        self.btn_crosshair.setGeometry(QtCore.QRect(290, 25, 81, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_crosshair.setFont(font)
        self.btn_crosshair.setObjectName("btn_crosshair")
        self.groupBox_2 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_2.setGeometry(QtCore.QRect(710, 5, 71, 91))
        self.groupBox_2.setObjectName("groupBox_2")
        self.btn_strechLow = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechLow.setGeometry(QtCore.QRect(10, 20, 61, 21))
        self.btn_strechLow.setObjectName("btn_strechLow")
        self.btn_strechMid = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechMid.setGeometry(QtCore.QRect(10, 40, 61, 21))
        self.btn_strechMid.setObjectName("btn_strechMid")
        self.btn_strechHigh = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechHigh.setGeometry(QtCore.QRect(10, 60, 61, 21))
        self.btn_strechHigh.setObjectName("btn_strechHigh")
        self.groupBox_3 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_3.setGeometry(QtCore.QRect(630, 5, 71, 91))
        self.groupBox_3.setObjectName("groupBox_3")
        self.btn_size25 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size25.setGeometry(QtCore.QRect(10, 20, 51, 21))
        self.btn_size25.setObjectName("btn_size25")
        self.btn_size50 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size50.setGeometry(QtCore.QRect(10, 40, 51, 21))
        self.btn_size50.setObjectName("btn_size50")
        self.btn_size100 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size100.setGeometry(QtCore.QRect(10, 60, 51, 21))
        self.btn_size100.setObjectName("btn_size100")
        self.btn_solve = QtWidgets.QPushButton(ImageDialog)
        self.btn_solve.setGeometry(QtCore.QRect(290, 60, 81, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_solve.setFont(font)
        self.btn_solve.setObjectName("btn_solve")
        self.groupBox_4 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_4.setGeometry(QtCore.QRect(540, 5, 81, 91))
        self.groupBox_4.setObjectName("groupBox_4")
        self.btn_colorGrey = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorGrey.setGeometry(QtCore.QRect(10, 20, 61, 21))
        self.btn_colorGrey.setObjectName("btn_colorGrey")
        self.btn_colorCool = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorCool.setGeometry(QtCore.QRect(10, 40, 61, 21))
        self.btn_colorCool.setObjectName("btn_colorCool")
        self.btn_colorRainbow = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorRainbow.setGeometry(QtCore.QRect(10, 60, 71, 21))
        self.btn_colorRainbow.setObjectName("btn_colorRainbow")
        self.btn_startContExposures = QtWidgets.QPushButton(ImageDialog)
        self.btn_startContExposures.setGeometry(QtCore.QRect(100, 10, 81, 81))
        self.btn_startContExposures.setObjectName("btn_startContExposures")
        self.btn_stopContExposures = QtWidgets.QPushButton(ImageDialog)
        self.btn_stopContExposures.setGeometry(QtCore.QRect(190, 10, 81, 81))
        self.btn_stopContExposures.setObjectName("btn_stopContExposures")
        self.label_82 = QtWidgets.QLabel(ImageDialog)
        self.label_82.setGeometry(QtCore.QRect(290, 5, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_82.setFont(font)
        self.label_82.setObjectName("label_82")
        self.line_14 = QtWidgets.QFrame(ImageDialog)
        self.line_14.setGeometry(QtCore.QRect(340, 5, 31, 21))
        self.line_14.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_14.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_14.setObjectName("line_14")
        self.cross4 = QtWidgets.QFrame(ImageDialog)
        self.cross4.setGeometry(QtCore.QRect(395, 385, 21, 241))
        self.cross4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.cross4.setFrameShape(QtWidgets.QFrame.VLine)
        self.cross4.setObjectName("cross4")
        self.cross2 = QtWidgets.QFrame(ImageDialog)
        self.cross2.setGeometry(QtCore.QRect(395, 114, 21, 241))
        self.cross2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.cross2.setFrameShape(QtWidgets.QFrame.VLine)
        self.cross2.setObjectName("cross2")
        self.cross1 = QtWidgets.QFrame(ImageDialog)
        self.cross1.setGeometry(QtCore.QRect(10, 360, 381, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.cross1.setFont(font)
        self.cross1.setFrameShadow(QtWidgets.QFrame.Plain)
        self.cross1.setFrameShape(QtWidgets.QFrame.HLine)
        self.cross1.setObjectName("cross1")
        self.cross3 = QtWidgets.QFrame(ImageDialog)
        self.cross3.setGeometry(QtCore.QRect(420, 360, 361, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.cross3.setFont(font)
        self.cross3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.cross3.setFrameShape(QtWidgets.QFrame.HLine)
        self.cross3.setObjectName("cross3")
        self.le_INDICameraStatus = QtWidgets.QLineEdit(ImageDialog)
        self.le_INDICameraStatus.setGeometry(QtCore.QRect(390, 60, 121, 31))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.le_INDICameraStatus.setFont(font)
        self.le_INDICameraStatus.setMouseTracking(False)
        self.le_INDICameraStatus.setAcceptDrops(False)
        self.le_INDICameraStatus.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_INDICameraStatus.setMaxLength(15)
        self.le_INDICameraStatus.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_INDICameraStatus.setReadOnly(False)
        self.le_INDICameraStatus.setObjectName("le_INDICameraStatus")

        self.retranslateUi(ImageDialog)
        QtCore.QMetaObject.connectSlotsByName(ImageDialog)

    def retranslateUi(self, ImageDialog):
        _translate = QtCore.QCoreApplication.translate
        ImageDialog.setWindowTitle(_translate("ImageDialog", "Image"))
        self.btn_expose.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Single exposure</p></body></html>"))
        self.btn_expose.setText(_translate("ImageDialog", "Expose\n"
"single\n"
"Shot"))
        self.btn_crosshair.setText(_translate("ImageDialog", "Crosshairs"))
        self.groupBox_2.setTitle(_translate("ImageDialog", "Strech"))
        self.btn_strechLow.setText(_translate("ImageDialog", "Low"))
        self.btn_strechMid.setText(_translate("ImageDialog", "Mid"))
        self.btn_strechHigh.setText(_translate("ImageDialog", "High"))
        self.groupBox_3.setTitle(_translate("ImageDialog", "Zoom"))
        self.btn_size25.setText(_translate("ImageDialog", "25%"))
        self.btn_size50.setText(_translate("ImageDialog", "50%"))
        self.btn_size100.setText(_translate("ImageDialog", "100%"))
        self.btn_solve.setText(_translate("ImageDialog", "Solve"))
        self.groupBox_4.setTitle(_translate("ImageDialog", "Colors"))
        self.btn_colorGrey.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme black /white</p></body></html>"))
        self.btn_colorGrey.setText(_translate("ImageDialog", "Grey"))
        self.btn_colorCool.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme red/blue</p></body></html>"))
        self.btn_colorCool.setText(_translate("ImageDialog", "Cool"))
        self.btn_colorRainbow.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme rainbow</p></body></html>"))
        self.btn_colorRainbow.setText(_translate("ImageDialog", "Rainbow"))
        self.btn_startContExposures.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Starting taking continously exposures</p></body></html>"))
        self.btn_startContExposures.setText(_translate("ImageDialog", "Start\n"
"cont.\n"
"Exposures"))
        self.btn_stopContExposures.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Stopping taking continously exposures</p></body></html>"))
        self.btn_stopContExposures.setText(_translate("ImageDialog", "Stop\n"
"cont.\n"
"Exposures"))
        self.label_82.setText(_translate("ImageDialog", "Views"))
        self.le_INDICameraStatus.setToolTip(_translate("ImageDialog", "<html><head/><body><p>IP address of relay switch</p></body></html>"))
        self.le_INDICameraStatus.setText(_translate("ImageDialog", "---"))




############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
import logging
from baseclasses.camera import MWCamera
import indi.indi_xml as indiXML
import PyQt5


class INDICamera(MWCamera):
    logger = logging.getLogger(__name__)                                                                                    # logging enabling

    def __init__(self, app):
        self.app = app
        self.appRunning = False
        self.cameraConnected = False
        self.cameraStatus = ''
        self.appInstallPath = ''
        self.appAvailable = False
        self.appName = 'INDI Camera'
        self.appExe = ''
        self.tryConnectionCounter = 0
        self.imagingStarted = False
        self.checkAppInstall()

    def checkAppInstall(self):
        self.appAvailable = False

    def checkAppStatus(self):
        if self.app.INDIworker.connected:
            self.tryConnectionCounter = 0
            self.appRunning = True
        else:
            self.tryConnectionCounter += 1
            self.appRunning = False
            self.cameraConnected = False
            if self.tryConnectionCounter < 3:
                self.logger.warning('{0} is not running'.format(self.appName))
            elif self.tryConnectionCounter == 3:
                self.logger.error('No connection to {0} possible - stop logging this connection error'.format(self.appName))

    def connectCamera(self):
        if self.appRunning and self.app.INDIworker.driverNameCCD != '':
            self.app.INDISendCommandQueue.put(indiXML.newSwitchVector([indiXML.oneSwitch('On', indi_attr={'name': 'CONNECT'})], indi_attr={'name': 'CONNECTION', 'device': self.app.INDIworker.driverNameCCD}))

    def disconnectCamera(self):
        if self.cameraConnected:
            self.app.INDISendCommandQueue.put(indiXML.newSwitchVector([indiXML.oneSwitch('Off', indi_attr={'name': 'CONNECT'})], indi_attr={'name': 'CONNECTION', 'device': self.app.INDIworker.driverNameCCD}))

    def getImage(self, modelData):
        binning = int(float(modelData['binning']))
        exposureLength = int(float(modelData['exposure']))
        speed = modelData['speed']
        filename = modelData['file']
        path = modelData['base_dir_images']
        imagePath = path + '/' + filename
        self.app.INDIworker.imagePath = imagePath
        if self.cameraConnected and self.app.INDIworker.driverNameCCD != '':
            self.app.INDIworker.receivedImage = False
            # Enable BLOB mode.
            self.app.INDISendCommandQueue.put(indiXML.enableBLOB('Also', indi_attr={'device': self.app.INDIworker.driverNameCCD}))
            # set to raw - no compression mode
            self.app.INDISendCommandQueue.put(indiXML.newSwitchVector([indiXML.oneSwitch('On', indi_attr={'name': 'CCD_RAW'})], indi_attr={'name': 'CCD_COMPRESSION', 'device': self.app.INDIworker.driverNameCCD}))
            # set frame type
            self.app.INDISendCommandQueue.put(indiXML.newSwitchVector([indiXML.oneSwitch('On', indi_attr={'name': 'FRAME_LIGHT'})], indi_attr={'name': 'CCD_FRAME_TYPE', 'device': self.app.INDIworker.driverNameCCD}))
            # set binning
            self.app.INDISendCommandQueue.put(indiXML.newNumberVector([indiXML.oneNumber(binning, indi_attr={'name': 'HOR_BIN'}), indiXML.oneNumber(binning, indi_attr={'name': 'VER_BIN'})], indi_attr={'name': 'CCD_BINNING', 'device': self.app.INDIworker.driverNameCCD}))
            # Request image.
            self.app.INDISendCommandQueue.put(indiXML.newNumberVector([indiXML.oneNumber(exposureLength, indi_attr={'name': 'CCD_EXPOSURE_VALUE'})], indi_attr={'name': 'CCD_EXPOSURE', 'device': self.app.INDIworker.driverNameCCD}))
            self.imagingStarted = True
            while not self.app.INDIworker.receivedImage:
                PyQt5.QtWidgets.QApplication.processEvents()
        modelData['imagepath'] = self.app.INDIworker.imagePath
        return True, 'OK', modelData

    def solveImage(self, modelData):
        pass

    def getCameraProps(self):
        if self.cameraConnected and self.app.INDIworker.driverNameCCD != '':
            sizeX = self.app.INDIworker.device[self.app.INDIworker.driverNameCCD]['CCD_INFO']['CCD_MAX_X']
            sizeY = self.app.INDIworker.device[self.app.INDIworker.driverNameCCD]['CCD_INFO']['CCD_MAX_Y']
            return True, 'OK', sizeX, sizeY, False, 'High'
        else:
            return False, 'Camera not connected', 0, 0, False, ''

    def getCameraStatus(self):
        if self.appRunning and self.app.INDIworker.driverNameCCD != '':
            if self.app.INDIworker.device[self.app.INDIworker.driverNameCCD]['CONNECTION']['CONNECT'] == 'On':
                self.cameraConnected = True
            else:
                self.cameraConnected = False
                self.cameraStatus = 'DISCONNECTED'
            if self.cameraConnected:
                if float(self.app.INDIworker.device[self.app.INDIworker.driverNameCCD]['CCD_EXPOSURE']['CCD_EXPOSURE_VALUE']):
                    self.cameraStatus = 'INTEGRATING'
                else:
                    self.cameraStatus = 'READY - IDLE'
        else:
            self.cameraStatus = 'ERROR'
            self.cameraConnected = False
        self.app.INDIDataQueue.put({'Name': 'CameraStatus', 'value': self.cameraStatus})



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
import platform
import json
import logging
import socket
import timeit
from baseclasses.camera import MWCamera


class TheSkyX(MWCamera):
    logger = logging.getLogger(__name__)

    def __init__(self, app):
        super(TheSkyX, self).__init__(app)
        self.host = '127.0.0.1'
        self.port = 3040
        self.responseSuccess = '|No error. Error = 0.'
        self.appExe = 'TheSkyX.exe'
        self.checkAppInstall()

    def checkAppInstall(self):
        if platform.system() == 'Windows':
            self.appAvailable, self.appName, self.appInstallPath = self.app.checkRegistrationKeys('TheSkyX')
        else:
            self.appAvailable = True
            self.appName = 'TheSkyX'
            self.appInstallPath = ''
        if self.appAvailable:
            self.app.messageQueue.put('Found: {0}'.format(self.appName))
            self.logger.info('Name: {0}, Path: {1}'.format(self.appName, self.appInstallPath))
        else:
            self.logger.info('Application TheSkyX not found on computer')

    def connectCamera(self):
        if self.appRunning:
            try:
                tsxSocket = socket.socket()
                tsxSocket.connect((self.host, self.port))
                command = '/* Java Script */ '
                command += 'ccdsoftCamera.Connect();'
                self.cameraConnected, response = self.sendCommand(command)
            except Exception as e:
                self.logger.error('error: {0}'.format(e))
                self.cameraConnected = False
            finally:
                # noinspection PyUnboundLocalVariable
                tsxSocket.close()

    def disconnectCamera(self):
        if self.appRunning:
            try:
                tsxSocket = socket.socket()
                tsxSocket.connect((self.host, self.port))
                command = '/* Java Script */ '
                command += 'ccdsoftCamera.Disconnect();'
                self.sendCommand(command)
                self.cameraConnected = False
            except Exception as e:
                self.logger.error('error: {0}'.format(e))
            finally:
                # noinspection PyUnboundLocalVariable
                tsxSocket.close()

    def sendCommand(self, command):
        try:
            tsxSocket = socket.socket()
            tsxSocket.connect((self.host, self.port))
            tsxSocket.send(command.encode())
            response = str(tsxSocket.recv(1024).decode())

            if response.endswith(self.responseSuccess):
                response = response.replace(self.responseSuccess, '')
                return True, response
            else:
                return False, response
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, format(e)
        finally:
            # noinspection PyUnboundLocalVariable
            tsxSocket.close()

    def solveImage(self, modelData):
        if modelData['Blind']:
            self.logger.warning('Blind mode is not supported. TheSkyX allows to switch to All Sky Image Link by scripting. Please, enable All Sky Image Link manually in TheSkyX')

        try:
            command = '/* Java Script */'
            command += 'ccdsoftCamera.Asynchronous=0;'
            command += 'ImageLink.pathToFITS="' + str(modelData['ImagePath']).replace('\\', '/') + '";'
            if modelData['ScaleHint']:
                command += 'ImageLink.unknownScale=0;'
                command += 'ImageLink.scale=' + str(modelData['ScaleHint']) + ';'
            else:
                command += 'ImageLink.unknownScale=1;'
                command += 'ImageLink.scale=2;'
            command += 'ImageLink.execute();'
            command += 'Out=String(\'{"succeeded":"\'+ImageLinkResults.succeeded+\'","imageCenterRAJ2000":"\'+ImageLinkResults.imageCenterRAJ2000+\'","imageCenterDecJ2000":"\'+ImageLinkResults.imageCenterDecJ2000+\'","imageScale":"\'+ImageLinkResults.imageScale+\'","imagePositionAngle":"\'+ImageLinkResults.imagePositionAngle+\'"}\');'

            startTime = timeit.default_timer()
            success, response = self.sendCommand(command)
            solveTime = timeit.default_timer() - startTime

            if success:
                captureResponse = json.loads(response)

                if captureResponse['succeeded'] == '1':
                    modelData['RaJ2000Solved'] = float(captureResponse['imageCenterRAJ2000'])
                    modelData['DecJ2000Solved'] = float(captureResponse['imageCenterDecJ2000'])
                    modelData['Scale'] = float(captureResponse['imageScale'])
                    modelData['Angle'] = float(captureResponse['imagePositionAngle'])
                    modelData['TimeTS'] = solveTime
                    return True, 'Solved', modelData
                else:
                    return False, 'Unsolved', modelData
            else:
                return False, 'Request failed', modelData
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', modelData

    def getImage(self, modelData):
        # TODO: how is TSX dealing with ISO settings for DSLR?

        try:
            command = '/* Java Script */'
            command += 'ccdsoftCamera.Asynchronous=0;'
            if modelData['CanSubframe']:
                command += 'ccdsoftCamera.Subframe=1;'
                command += 'ccdsoftCamera.SubframeLeft=' + str(modelData['OffX']) + ';'
                command += 'ccdsoftCamera.SubframeTop=' + str(modelData['OffY']) + ';'
                command += 'ccdsoftCamera.SubframeRight=' + str(modelData['OffX'] + modelData['SizeX']) + ';'
                command += 'ccdsoftCamera.SubframeBottom=' + str(modelData['OffY'] + modelData['SizeY']) + ';'
            else:
                command += 'ccdsoftCamera.Subframe=0;'

            if modelData['Speed'] == 'HiSpeed':
                command += 'ccdsoftCamera.setPropStr("m_csExCameraMode", "Fast Image Download");'
            else:
                command += 'ccdsoftCamera.setPropStr("m_csExCameraMode", "High Image Quality");'

            command += 'ccdsoftCamera.BinX='+str(modelData['Binning'])+';'
            command += 'ccdsoftCamera.BinY='+str(modelData['Binning'])+';'
            command += 'ccdsoftCamera.ExposureTime='+str(modelData['Exposure'])+';'
            command += 'ccdsoftCamera.AutoSavePath="'+str(modelData['BaseDirImages'])+'";'
            command += 'ccdsoftCamera.AutoSaveOn=1;'
            command += 'ccdsoftCamera.Frame="cdLight";'
            command += 'ccdsoftCamera.TakeImage();'
            command += 'var Out = "";'
            command += 'Out=ccdsoftCamera.LastImageFileName;'
            success, response = self.sendCommand(command)

            modelData['ImagePath'] = response

            return success, response, modelData
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', ''

    def getCameraStatus(self):
        if self.appRunning:
            try:
                command = '/* Java Script */'
                command += 'var Out = "";'
                command += 'ccdsoftCamera.Asynchronous=0;'
                command += 'Out=ccdsoftCamera.ExposureStatus;'
                success, response = self.sendCommand(command)
                self.cameraConnected = True
                if response == 'Not Connected':
                    response = 'DISCONNECTED'
                    self.cameraConnected = False
                elif response == 'Ready':
                    response = 'READY - IDLE'
                elif 'Exposing' in response:
                    response = 'INTEGRATING'
                self.cameraStatus = response
            except Exception as e:
                self.logger.error('error: {0}'.format(e))
            finally:
                pass

    def getCameraProps(self):
        # TODO: Get the chance to implement subframe on / off if a camera doesn't support this
        # TODO: Gain setting in CMOS Cameras necessary ?
        try:
            command = '/* Java Script */'
            command += 'var Out = "";'
            command += 'ccdsoftCamera.Asynchronous=0;'
            command += 'Out=String(\'{"WidthInPixels":"\'+ccdsoftCamera.WidthInPixels+\'","HeightInPixels":"\'+ccdsoftCamera.HeightInPixels+\'"}\');'
            success, response = self.sendCommand(command)
            captureResponse = json.loads(response)
            return success, '', int(captureResponse['WidthInPixels']), int(captureResponse['HeightInPixels']), True, 'Not Set'
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', '', '', '', ''
